@@ -782,7 +782,20 @@ int arch_vcpu_init(struct vmm_vcpu *vcpu)
 
 	memset(arm_regs(vcpu), 0, sizeof(arch_regs_t));
 	arm_regs(vcpu)->pc = vcpu->start_pc;
-	arm_regs(vcpu)->sp_excp = vcpu->stack_va + vcpu->stack_sz - 4;
+
+	/*
+	 * Stacks must be 64-bits aligned to respect AAPCS:
+	 * Procedure Call Standard for the ARM Architecture.
+	 * To do so, AAPCS advises that all SP must be set to
+	 * a value which is 0 modulo 8.
+	 * The compiler takes care of the frame size.
+	 *
+	 * This is terribly important because it messes runtime
+	 * with values greater than 32 bits (e.g. 64-bits integers).
+	 */
+	arm_regs(vcpu)->sp_excp = vcpu->stack_va + vcpu->stack_sz - 8;
+	arm_regs(vcpu)->sp_excp = arm_regs(vcpu)->sp_excp & ~0x7;
+
 	if (vcpu->is_normal) {
 		arm_regs(vcpu)->cpsr  = CPSR_ZERO_MASK;
 		arm_regs(vcpu)->cpsr |= CPSR_ASYNC_ABORT_DISABLED;
