@@ -24,6 +24,7 @@
 #define __CPU_INLINE_ASM_H__
 
 #include <vmm_types.h>
+#include <cpu_defines.h>
 
 #if defined(CONFIG_ARMV5)
 
@@ -83,22 +84,76 @@ static inline u16 rev16(u16 v)
 /* FIXME: */
 #define clrex()
 
+#elif defined(CONFIG_ARMV6)
+
+#define ldrex(addr, data)	asm volatile("ldrex	%0, [%1]\n\t" \
+				: "=r"(data) : "r"(addr))
+
+#define strex(addr, data, res)	asm volatile("strex	%0, %1, [%2]\n\t" \
+				: "=&r"(res) : "r"(data), "r"(addr))
+
+#define clrex()	
+
 #else
 
 #define ldrex(addr, data)	asm volatile("ldrex	%0, [%1]\n\t" \
 				: "=r"(data) : "r"(addr))
 
 #define strex(addr, data, res)	asm volatile("strex	%0, %1, [%2]\n\t" \
-				: "=r"(res) : "r"(data), "r"(addr))
+				: "=&r"(res) : "r"(data), "r"(addr))
 
 #define clrex()			asm volatile("clrex\n\t")
 
 #endif
 
+/* General CP14 Register Read/Write */
+
+#define read_teecr()		({ u32 rval; asm volatile(\
+				" mrc     p14, 6, %0, c0, c0, 0\n\t" \
+				: "=r" (rval) : : "memory", "cc"); rval;})
+
+#define write_teecr(val)	asm volatile(\
+				" mcr     p14, 6, %0, c0, c0, 0\n\t" \
+				:: "r" ((val)) : "memory", "cc")
+
+#define read_teehbr()		({ u32 rval; asm volatile(\
+				" mrc     p14, 6, %0, c1, c0, 0\n\t" \
+				: "=r" (rval) : : "memory", "cc"); rval;})
+
+#define write_teehbr(val)	asm volatile(\
+				" mcr     p14, 6, %0, c1, c0, 0\n\t" \
+				:: "r" ((val)) : "memory", "cc")
+
+/* General CP15 Register Read/Write */
+
 #define read_midr()		({ u32 rval; asm volatile(\
 				" mrc     p15, 0, %0, c0, c0, 0\n\t" \
 				: "=r" (rval) : : "memory", "cc"); rval;})
 
+#define read_ctr()		({ u32 rval; asm volatile(\
+				" mrc     p15, 0, %0, c0, c0, 1\n\t" \
+				: "=r" (rval) : : "memory", "cc"); rval;})
+
+#define read_mpidr()		({ u32 rval; asm volatile(\
+				" mrc     p15, 0, %0, c0, c0, 5\n\t" \
+				: "=r" (rval) : : "memory", "cc"); rval;})
+
+#define read_ccsidr()		({ u32 rval; asm volatile(\
+				" mrc     p15, 1, %0, c0, c0, 0\n\t" \
+				: "=r" (rval) : : "memory", "cc"); rval;})
+
+#define read_clidr()		({ u32 rval; asm volatile(\
+				" mrc     p15, 1, %0, c0, c0, 1\n\t" \
+				: "=r" (rval) : : "memory", "cc"); rval;})
+
+#define read_csselr()		({ u32 rval; asm volatile(\
+				" mrc     p15, 2, %0, c0, c0, 0\n\t" \
+				: "=r" (rval) : : "memory", "cc"); rval;})
+
+#define write_csselr(val)	asm volatile(\
+				" mcr     p15, 2, %0, c0, c0, 0\n\t" \
+				:: "r" ((val)) : "memory", "cc")
+
 #define read_pfr0()		({ u32 rval; asm volatile(\
 				" mrc     p15, 0, %0, c0, c1, 0\n\t" \
 				: "=r" (rval) : : "memory", "cc"); rval;})
@@ -155,30 +210,6 @@ static inline u16 rev16(u16 v)
 				" mrc     p15, 0, %0, c0, c2, 5\n\t" \
 				: "=r" (rval) : : "memory", "cc"); rval;})
 
-#define read_ccsidr()		({ u32 rval; asm volatile(\
-				" mrc     p15, 1, %0, c0, c0, 0\n\t" \
-				: "=r" (rval) : : "memory", "cc"); rval;})
-
-#define read_csselr()		({ u32 rval; asm volatile(\
-				" mrc     p15, 2, %0, c0, c0, 0\n\t" \
-				: "=r" (rval) : : "memory", "cc"); rval;})
-
-#define write_csselr(val)	asm volatile(\
-				" mcr     p15, 2, %0, c0, c0, 0\n\t" \
-				:: "r" ((val)) : "memory", "cc")
-
-#define read_clidr()		({ u32 rval; asm volatile(\
-				" mrc     p15, 1, %0, c0, c0, 1\n\t" \
-				: "=r" (rval) : : "memory", "cc"); rval;})
-
-#define read_ctr()		({ u32 rval; asm volatile(\
-				" mrc     p15, 0, %0, c0, c0, 1\n\t" \
-				: "=r" (rval) : : "memory", "cc"); rval;})
-
-#define read_mpidr()		({ u32 rval; asm volatile(\
-				" mrc     p15, 0, %0, c0, c0, 5\n\t" \
-				: "=r" (rval) : : "memory", "cc"); rval;})
-
 #define read_sctlr()		({ u32 rval; asm volatile(\
 				" mrc     p15, 0, %0, c1, c0, 0\n\t" \
 				: "=r" (rval) : : "memory", "cc"); rval;})
@@ -187,6 +218,32 @@ static inline u16 rev16(u16 v)
 				" mcr     p15, 0, %0, c1, c0, 0\n\t" \
 				:: "r" ((val)) : "memory", "cc")
 
+#define read_actlr()		({ u32 rval; asm volatile(\
+				" mrc     p15, 0, %0, c1, c0, 1\n\t" \
+				: "=r" (rval) : : "memory", "cc"); rval;})
+
+#define write_actlr(val)	asm volatile(\
+				" mcr     p15, 0, %0, c1, c0, 1\n\t" \
+				:: "r" ((val)) : "memory", "cc")
+
+#ifndef CONFIG_ARMV5
+
+#define read_cpacr()		({ u32 rval; asm volatile(\
+				" mrc     p15, 0, %0, c1, c0, 2\n\t" \
+				: "=r" (rval) : : "memory", "cc"); rval;})
+
+#define write_cpacr(val)	asm volatile(\
+				" mcr     p15, 0, %0, c1, c0, 2\n\t" \
+				:: "r" ((val)) : "memory", "cc")
+
+#else
+
+#define read_cpacr()		0
+
+#define write_cpacr(val)
+
+#endif
+
 #define read_dacr()		({ u32 rval; asm volatile(\
 				" mrc     p15, 0, %0, c3, c0, 0\n\t" \
 				: "=r" (rval) : : "memory", "cc"); rval;})
@@ -211,7 +268,7 @@ static inline u16 rev16(u16 v)
 				" mcr     p15, 0, %0, c2, c0, 1\n\t" \
 				:: "r" ((val)) : "memory", "cc")
 
-#if defined(CONFIG_ARMV7A_SECUREX)
+#ifndef CONFIG_ARMV5
 
 #define read_vbar()		({ u32 rval; asm volatile(\
 				" mrc     p15, 0, %0, c12, c0, 0\n\t" \
@@ -221,7 +278,12 @@ static inline u16 rev16(u16 v)
 				" mcr     p15, 0, %0, c12, c0, 0\n\t" \
 				:: "r" ((val)) : "memory", "cc")
 
-#endif /* CONFIG_ARMV7A_SECUREX */
+#else
+
+#define read_vbar()		0
+#define write_vbar(val)
+
+#endif /* CONFIG_ARMV7A */
 
 #define read_ttbcr()		({ u32 rval; asm volatile(\
 				" mrc     p15, 0, %0, c2, c0, 2\n\t" \
@@ -311,19 +373,11 @@ extern unsigned int *_ifar;
 				:: "r" ((val)) : "memory", "cc")
 #endif
 
-#define invalid_tlb()		({ u32 rval=0; asm volatile(\
-				" mcr     p15, 0, %0, c8, c7, 0\n\t" \
-				: "=r" (rval) : : "memory", "cc"); rval;})
-
-#define invalid_tlb_line(va)	asm volatile(\
-				" mcr     p15, 0, %0, c8, c7, 1\n\t" \
-				:: "r" ((va)) : "memory", "cc")
-
 #define invalid_i_tlb()		({ u32 rval=0; asm volatile(\
 				" mcr     p15, 0, %0, c8, c5, 0\n\t" \
 				: "=r" (rval) : : "memory", "cc"); rval;})
 
-#define invalid_i_tlb_line(va)	asm volatile(\
+#define invalid_i_tlb_mva(va)	asm volatile(\
 				" mcr     p15, 0, %0, c8, c5, 1\n\t" \
 				:: "r" ((va)) : "memory", "cc")
 
@@ -331,10 +385,26 @@ extern unsigned int *_ifar;
 				" mcr     p15, 0, %0, c8, c6, 0\n\t" \
 				: "=r" (rval) : : "memory", "cc"); rval;})
 
-#define invalid_d_tlb_line(va)	asm volatile(\
+#define invalid_d_tlb_mva(va)	asm volatile(\
 				" mcr     p15, 0, %0, c8, c6, 1\n\t" \
 				:: "r" ((va)) : "memory", "cc")
 
+#define invalid_tlb()		({ u32 rval=0; asm volatile(\
+				" mcr     p15, 0, %0, c8, c7, 0\n\t" \
+				: "=r" (rval) : : "memory", "cc"); rval;})
+
+#define invalid_tlb_mva(va)	asm volatile(\
+				" mcr     p15, 0, %0, c8, c7, 1\n\t" \
+				:: "r" ((va)) : "memory", "cc")
+
+#if !defined(CONFIG_ARMV5)
+
+#define invalid_tlb_asid(asid)	asm volatile(\
+				" mcr     p15, 0, %0, c8, c7, 2\n\t" \
+				:: "r" ((asid)) : "memory", "cc")
+
+#endif
+
 #define read_contextidr()	({ u32 rval; asm volatile(\
 				" mrc     p15, 0, %0, c13, c0, 1\n\t" \
 				: "=r" (rval) : : "memory", "cc"); rval;})
@@ -384,4 +454,84 @@ extern unsigned int *_ifar;
 				:: "r" ((val)) : "memory", "cc")
 #endif
 
+/* VFP Control Register Read/Write */
+#define read_fpexc()		({ u32 rval; asm volatile(\
+				" mrc p10, 7, %0, c8, c0, 0\n\t" \
+				: "=r" (rval) : : "memory", "cc"); rval;})
+
+#define write_fpexc(val)	asm volatile(\
+				" mcr p10, 7, %0, c8, c0, 0\n\t" \
+				:: "r" ((val)) : "memory", "cc")
+
+#define read_fpscr()		({ u32 rval; asm volatile(\
+				" mrc p10, 7, %0, c1, c0, 0\n\t" \
+				: "=r" (rval) : : "memory", "cc"); rval;})
+
+#define write_fpscr(val)	asm volatile(\
+				" mcr p10, 7, %0, c1, c0, 0\n\t" \
+				:: "r" ((val)) : "memory", "cc")
+
+#define read_fpsid()		({ u32 rval; asm volatile(\
+				" mrc p10, 7, %0, c0, c0, 0\n\t" \
+				: "=r" (rval) : : "memory", "cc"); rval;})
+
+#define write_fpsid(val)	asm volatile(\
+				" mcr p10, 7, %0, c0, c0, 0\n\t" \
+				:: "r" ((val)) : "memory", "cc")
+
+#define read_fpinst()		({ u32 rval; asm volatile(\
+				" mrc p10, 7, %0, c9, c0, 0\n\t" \
+				: "=r" (rval) : : "memory", "cc"); rval;})
+
+#define write_fpinst(val)	asm volatile(\
+				" mcr p10, 7, %0, c9, c0, 0\n\t" \
+				:: "r" ((val)) : "memory", "cc")
+
+#define read_fpinst2()		({ u32 rval; asm volatile(\
+				" mrc p10, 7, %0, c10, c0, 0\n\t" \
+				: "=r" (rval) : : "memory", "cc"); rval;})
+
+#define write_fpinst2(val)	asm volatile(\
+				" mcr p10, 7, %0, c10, c0, 0\n\t" \
+				:: "r" ((val)) : "memory", "cc")
+
+#define read_mvfr0()		({ u32 rval; asm volatile(\
+				" mrc p10, 7, %0, c7, c0, 0\n\t" \
+				: "=r" (rval) : : "memory", "cc"); rval;})
+
+#define write_mvfr0(val)	asm volatile(\
+				" mcr p10, 7, %0, c7, c0, 0\n\t" \
+				:: "r" ((val)) : "memory", "cc")
+
+#define read_mvfr1()		({ u32 rval; asm volatile(\
+				" mrc p10, 7, %0, c6, c0, 0\n\t" \
+				: "=r" (rval) : : "memory", "cc"); rval;})
+
+#define write_mvfr1(val)	asm volatile(\
+				" mcr p10, 7, %0, c6, c0, 0\n\t" \
+				:: "r" ((val)) : "memory", "cc")
+
+/* CPU feature checking macros */
+
+#ifndef CONFIG_ARMV5
+
+#define cpu_supports_thumbee()	(((read_pfr0() & ID_PFR0_STATE3_MASK) \
+					>> ID_PFR0_STATE3_SHIFT) == 0x1)
+
+#define cpu_supports_securex()	(read_pfr1() & ID_PFR1_SECUREX_MASK)
+
+#else
+
+#define cpu_supports_thumbee()	0
+
+#define cpu_supports_securex()	0
+
+#endif
+
+#ifdef CONFIG_ARMV5
+#define cpu_supports_fpu()	0
+#else
+#define cpu_supports_fpu()	(!(read_fpsid() & FPSID_SW_MASK))
+#endif
+
 #endif
