@@ -34,9 +34,9 @@
 
 #include <arm_features.h>
 
-bool cpu_vcpu_cp15_read(struct vmm_vcpu *vcpu, 
+bool cpu_vcpu_cp15_read(struct vmm_vcpu *vcpu,
 			arch_regs_t *regs,
-			u32 opc1, u32 opc2, u32 CRn, u32 CRm, 
+			u32 opc1, u32 opc2, u32 CRn, u32 CRm,
 			u32 *data)
 {
 	struct arm_priv_cp15 *cp15 = &arm_priv(vcpu)->cp15;
@@ -570,6 +570,11 @@ int cpu_vcpu_cp15_init(struct vmm_vcpu *vcpu, u32 cpuid)
 		cp15->c0_ccsid[2] = 0x711fe07a; /* 4096K L2 unified cache */
 		cp15->c1_sctlr = 0x00c50078;
 		break;
+	case ARM_CPUID_ARMV7:
+		cp15->c0_midr = read_midr();
+		cp15->c0_mpidr = vcpu->subid;
+		cp15->c1_sctlr = 0x00c50078;
+		break;
 	default:
 		cp15->c0_midr = cpuid;
 		cp15->c0_mpidr = vcpu->subid;
