@@ -24,6 +24,7 @@
 #define __CPU_INLINE_ASM_H__
 
 #include <vmm_types.h>
+#include <vmm_compiler.h>
 #include <cpu_defines.h>
 
 #define rev32(val)		({ u32 rval; asm volatile(\
@@ -47,6 +48,21 @@
 
 #define clrex()			asm volatile("clrex\n\t")
 
+#define __ACCESS_CP15(CRn, Op1, CRm, Op2)	\
+	"mrc", "mcr", stringify(p15, Op1, %0, CRn, CRm, Op2), u32
+#define __ACCESS_CP15_64(Op1, CRm)		\
+	"mrrc", "mcrr", stringify(p15, Op1, %Q0, %R0, CRm), u64
+
+#define __read_sysreg(r, w, c, t) ({				\
+	t __val;						\
+	asm volatile(r " " c : "=r" (__val));			\
+	__val;							\
+})
+#define read_sysreg(...)		__read_sysreg(__VA_ARGS__)
+
+#define __write_sysreg(v, r, w, c, t)	asm volatile(w " " c : : "r" ((t)(v)))
+#define write_sysreg(v, ...)		__write_sysreg(v, __VA_ARGS__)
+
 /* General CP14 Register Read/Write */
 
 #define read_teecr()		({ u32 rval; asm volatile(\
@@ -75,6 +91,10 @@
 				" mrc     p15, 0, %0, c0, c0, 5\n\t" \
 				: "=r" (rval) : : "memory", "cc"); rval;})
 
+#define read_midr()		({ u32 rval; asm volatile(\
+				" mrc     p15, 0, %0, c0, c0, 0\n\t" \
+				: "=r" (rval) : : "memory", "cc"); rval;})
+
 #define read_ccsidr()		({ u32 rval; asm volatile(\
 				" mrc     p15, 1, %0, c0, c0, 0\n\t" \
 				: "=r" (rval) : : "memory", "cc"); rval;})
