@@ -23,12 +23,62 @@
 #ifndef __CPU_DEFINES_H__
 #define __CPU_DEFINES_H__
 
-/* Maximum allowed VTLB entries */
-#define CPU_VCPU_VTLB_LINE_MASK				0x00000000
-#define CPU_VCPU_VTLB_LINE_SHIFT			0
-#define CPU_VCPU_VTLB_LINE_COUNT			1
-#define CPU_VCPU_VTLB_LINE_ENTRY_COUNT			128
-#define CPU_VCPU_VTLB_ENTRY_COUNT			128
+/*
+ * ARMv8 ARM reserves the following encoding for system registers:
+ * (Ref: ARMv8 ARM, Section: "System instruction class encoding overview",
+ *  C5.2, version:ARM DDI 0487A.f)
+ *	[20-19] : Op0
+ *	[18-16] : Op1
+ *	[15-12] : CRn
+ *	[11-8]  : CRm
+ *	[7-5]   : Op2
+ */
+#define sys_reg(op0, op1, crn, crm, op2) \
+	((((op0)&3)<<19)|((op1)<<16)|((crn)<<12)|((crm)<<8)|((op2)<<5))
+
+#define SYS_MIDR_EL1			sys_reg(3, 0, 0, 0, 0)
+#define SYS_MPIDR_EL1			sys_reg(3, 0, 0, 0, 5)
+#define SYS_REVIDR_EL1			sys_reg(3, 0, 0, 0, 6)
+
+#define SYS_ID_PFR0_EL1			sys_reg(3, 0, 0, 1, 0)
+#define SYS_ID_PFR1_EL1			sys_reg(3, 0, 0, 1, 1)
+#define SYS_ID_DFR0_EL1			sys_reg(3, 0, 0, 1, 2)
+#define SYS_ID_MMFR0_EL1		sys_reg(3, 0, 0, 1, 4)
+#define SYS_ID_MMFR1_EL1		sys_reg(3, 0, 0, 1, 5)
+#define SYS_ID_MMFR2_EL1		sys_reg(3, 0, 0, 1, 6)
+#define SYS_ID_MMFR3_EL1		sys_reg(3, 0, 0, 1, 7)
+
+#define SYS_ID_ISAR0_EL1		sys_reg(3, 0, 0, 2, 0)
+#define SYS_ID_ISAR1_EL1		sys_reg(3, 0, 0, 2, 1)
+#define SYS_ID_ISAR2_EL1		sys_reg(3, 0, 0, 2, 2)
+#define SYS_ID_ISAR3_EL1		sys_reg(3, 0, 0, 2, 3)
+#define SYS_ID_ISAR4_EL1		sys_reg(3, 0, 0, 2, 4)
+#define SYS_ID_ISAR5_EL1		sys_reg(3, 0, 0, 2, 5)
+#define SYS_ID_MMFR4_EL1		sys_reg(3, 0, 0, 2, 6)
+
+#define SYS_MVFR0_EL1			sys_reg(3, 0, 0, 3, 0)
+#define SYS_MVFR1_EL1			sys_reg(3, 0, 0, 3, 1)
+#define SYS_MVFR2_EL1			sys_reg(3, 0, 0, 3, 2)
+
+#define SYS_ID_AA64PFR0_EL1		sys_reg(3, 0, 0, 4, 0)
+#define SYS_ID_AA64PFR1_EL1		sys_reg(3, 0, 0, 4, 1)
+
+#define SYS_ID_AA64DFR0_EL1		sys_reg(3, 0, 0, 5, 0)
+#define SYS_ID_AA64DFR1_EL1		sys_reg(3, 0, 0, 5, 1)
+
+#define SYS_ID_AA64ISAR0_EL1		sys_reg(3, 0, 0, 6, 0)
+#define SYS_ID_AA64ISAR1_EL1		sys_reg(3, 0, 0, 6, 1)
+
+#define SYS_ID_AA64MMFR0_EL1		sys_reg(3, 0, 0, 7, 0)
+#define SYS_ID_AA64MMFR1_EL1		sys_reg(3, 0, 0, 7, 1)
+#define SYS_ID_AA64MMFR2_EL1		sys_reg(3, 0, 0, 7, 2)
+
+#define SYS_CNTFRQ_EL0			sys_reg(3, 3, 14, 0, 0)
+#define SYS_CTR_EL0			sys_reg(3, 3, 0, 0, 1)
+#define SYS_DCZID_EL0			sys_reg(3, 3, 0, 0, 7)
+
+#define REG_PSTATE_PAN_IMM		sys_reg(0, 0, 4, 0, 4)
+#define REG_PSTATE_UAO_IMM		sys_reg(0, 0, 4, 0, 3)
 
 /* Coprocessor related macros & defines */
 #define CPU_COPROC_COUNT				16
@@ -291,71 +341,75 @@
 /* HCR_EL2 */
 #define HCR_INITVAL					0x000000000
 #define HCR_ID_MASK					0x200000000
-#define HCR_ID_SHIFT					33         
+#define HCR_ID_SHIFT					33
 #define HCR_CD_MASK					0x100000000
-#define HCR_CD_SHIFT					32         
+#define HCR_CD_SHIFT					32
 #define HCR_RW_MASK					0x080000000
-#define HCR_RW_SHIFT					31         
+#define HCR_RW_SHIFT					31
 #define HCR_TRVM_MASK					0x040000000
-#define HCR_TRVM_SHIFT					30         
+#define HCR_TRVM_SHIFT					30
 #define HCR_HCD_MASK					0x020000000
-#define HCR_HCD_SHIFT					29         
+#define HCR_HCD_SHIFT					29
 #define HCR_TDZ_MASK					0x010000000
-#define HCR_TDZ_SHIFT					28         
+#define HCR_TDZ_SHIFT					28
 #define HCR_TGE_MASK					0x008000000
-#define HCR_TGE_SHIFT					27         
+#define HCR_TGE_SHIFT					27
 #define HCR_TVM_MASK					0x004000000
-#define HCR_TVM_SHIFT					26         
+#define HCR_TVM_SHIFT					26
 #define HCR_TTLB_MASK					0x002000000
-#define HCR_TTLB_SHIFT					25         
+#define HCR_TTLB_SHIFT					25
 #define HCR_TPU_MASK					0x001000000
-#define HCR_TPU_SHIFT					24         
+#define HCR_TPU_SHIFT					24
 #define HCR_TPC_MASK					0x000800000
-#define HCR_TPC_SHIFT					23         
+#define HCR_TPC_SHIFT					23
 #define HCR_TSW_MASK					0x000400000
-#define HCR_TSW_SHIFT					22         
+#define HCR_TSW_SHIFT					22
 #define HCR_TACR_MASK					0x000200000
-#define HCR_TACR_SHIFT					21         
+#define HCR_TACR_SHIFT					21
 #define HCR_TIDCP_MASK					0x000100000
-#define HCR_TIDCP_SHIFT					20         
+#define HCR_TIDCP_SHIFT					20
 #define HCR_TSC_MASK					0x000080000
-#define HCR_TSC_SHIFT					19         
+#define HCR_TSC_SHIFT					19
 #define HCR_TID3_MASK					0x000040000
-#define HCR_TID3_SHIFT					18         
+#define HCR_TID3_SHIFT					18
 #define HCR_TID2_MASK					0x000020000
-#define HCR_TID2_SHIFT					17         
+#define HCR_TID2_SHIFT					17
 #define HCR_TID1_MASK					0x000010000
-#define HCR_TID1_SHIFT					16         
+#define HCR_TID1_SHIFT					16
 #define HCR_TID0_MASK					0x000008000
-#define HCR_TID0_SHIFT					15         
+#define HCR_TID0_SHIFT					15
 #define HCR_TWE_MASK					0x000004000
-#define HCR_TWE_SHIFT					14         
+#define HCR_TWE_SHIFT					14
 #define HCR_TWI_MASK					0x000002000
-#define HCR_TWI_SHIFT					13         
+#define HCR_TWI_SHIFT					13
 #define HCR_DC_MASK					0x000001000
-#define HCR_DC_SHIFT					12         
+#define HCR_DC_SHIFT					12
 #define HCR_BSU_MASK					0x000000C00
-#define HCR_BSU_SHIFT					10         
+#define HCR_BSU_SHIFT					10
 #define HCR_FB_MASK					0x000000200
-#define HCR_FB_SHIFT					9          
+#define HCR_FB_SHIFT					9
 #define HCR_VSE_MASK					0x000000100
-#define HCR_VSE_SHIFT					8          
+#define HCR_VSE_SHIFT					8
 #define HCR_VI_MASK					0x000000080
-#define HCR_VI_SHIFT					7          
+#define HCR_VI_SHIFT					7
 #define HCR_VF_MASK					0x000000040
-#define HCR_VF_SHIFT					6          
+#define HCR_VF_SHIFT					6
 #define HCR_AMO_MASK					0x000000020
-#define HCR_AMO_SHIFT					5          
+#define HCR_AMO_SHIFT					5
 #define HCR_IMO_MASK					0x000000010
-#define HCR_IMO_SHIFT					4          
+#define HCR_IMO_SHIFT					4
 #define HCR_FMO_MASK					0x000000008
-#define HCR_FMO_SHIFT					3          
+#define HCR_FMO_SHIFT					3
 #define HCR_PTW_MASK					0x000000004
-#define HCR_PTW_SHIFT					2          
+#define HCR_PTW_SHIFT					2
 #define HCR_SWIO_MASK					0x000000002
-#define HCR_SWIO_SHIFT					1          
+#define HCR_SWIO_SHIFT					1
 #define HCR_VM_MASK					0x000000001
 #define HCR_VM_SHIFT					0
+#define HCR_DEFAULT_BITS				(HCR_AMO_MASK | \
+							 HCR_IMO_MASK | \
+							 HCR_FMO_MASK | \
+							 HCR_VM_MASK)
 
 /* CPTR_EL2 */
 #define CPTR_INITVAL					0x00000000LU
@@ -454,9 +508,11 @@
 #define ISS_COND_SHIFT					20
 
 /* MRS/MSR Trap ISS Encodings */
-#define ISS_SYSREG_ENC(op0, op2, op1, crn, crm)	(((op0) << 20) | \
-				((op2) << 17) | ((op1) << 14)  | \
-		 		((crn) << 10) | ((crm) << 1))	 
+#define ISS_SYSREG_ENC(op0, op2, op1, crn, crm)		(((op0) << 20) | \
+							 ((op2) << 17) | \
+							 ((op1) << 14)  | \
+							 ((crn) << 10) | \
+							 ((crm) << 1))
 
 #define ISS_SYSREG_MASK					0xfffffc1e
 #define ISS_RT_MASK					0x000003e0
@@ -500,6 +556,22 @@
 #define ISS_MCR_MRC_DIR_MASK				0x00000001
 #define ISS_MCR_MRC_DIR_SHIFT				0
 
+/* FPU exception ISS Encodings */
+#define ISS_FPEXEC_TFV_MASK				0x00800000
+#define ISS_FPEXEC_TFV_SHIFT				23
+#define ISS_FPEXEC_IDF_MASK				0x00000080
+#define ISS_FPEXEC_IDF_SHIFT				7
+#define ISS_FPEXEC_IXF_MASK				0x00000010
+#define ISS_FPEXEC_IXF_SHIFT				4
+#define ISS_FPEXEC_UFF_MASK				0x00000008
+#define ISS_FPEXEC_UFF_SHIFT				3
+#define ISS_FPEXEC_OFF_MASK				0x00000004
+#define ISS_FPEXEC_OFF_SHIFT				2
+#define ISS_FPEXEC_DZF_MASK				0x00000002
+#define ISS_FPEXEC_DZF_SHIFT				1
+#define ISS_FPEXEC_IOF_MASK				0x00000001
+#define ISS_FPEXEC_IOF_SHIFT				0
+
 /* Instruction/Data Abort ISS Encodings */
 #define ISS_ABORT_ISV_MASK				0x01000000
 #define ISS_ABORT_ISV_SHIFT				24
@@ -568,7 +640,7 @@
 #define TCR_TBI_SHIFT					20
 #define TCR_PS_MASK					0x00070000
 #define TCR_PS_SHIFT					16
-#define TCR_TG0_MASK					0x00004000
+#define TCR_TG0_MASK					0x0000c000
 #define TCR_TG0_SHIFT					14
 #define TCR_SH0_MASK					0x00003000
 #define TCR_SH0_SHIFT					12
@@ -598,7 +670,7 @@
 #define VTCR_INITVAL					0x80000000
 #define VTCR_PS_MASK					0x00070000
 #define VTCR_PS_SHIFT					16
-#define VTCR_TG0_MASK					0x00004000
+#define VTCR_TG0_MASK					0x0000c000
 #define VTCR_TG0_SHIFT					14
 #define VTCR_SH0_MASK					0x00003000
 #define VTCR_SH0_SHIFT					12
@@ -623,11 +695,7 @@
 #define VTCR_T0SZ_VAL(in_bits)				((64 - (in_bits)) & VTCR_T0SZ_MASK)
 
 /* MAIR_EL2 encodings */
-#define AINDEX_SO					0
-#define AINDEX_NORMAL_WT				1
-#define AINDEX_NORMAL_WB				2
-#define AINDEX_NORMAL_UC				3
-#define MAIR_INITVAL					0x0000000044FFBB00
+#define MAIR(val, idx)					((val) << ((idx) * 8))
 
 /* VPIDR */
 #define VPIDR_INITVAL					0x00000000
@@ -733,9 +801,9 @@
 #define ID_PFR1_PROGMODEL_SHIFT				0
 
 /* ID_AA64PFR0_EL1 */
-#define ID_AA64PFR0_ASIMD_MASK				0x00020000
-#define ID_AA64PFR0_ASIMD_SHIFT				17
-#define ID_AA64PFR0_FPU_MASK				0x00010000
+#define ID_AA64PFR0_ASIMD_MASK				0x00f00000
+#define ID_AA64PFR0_ASIMD_SHIFT				20
+#define ID_AA64PFR0_FPU_MASK				0x000f0000
 #define ID_AA64PFR0_FPU_SHIFT				16
 #define ID_AA64PFR0_EL3_A32				0x00002000
 #define ID_AA64PFR0_EL3_MASK				0x0000f000
