@@ -56,6 +56,7 @@ struct mmu_lpae_ctrl {
 	/* Initialized by memory read/write init */
 	struct cpu_ttbl *mem_rw_ttbl[CONFIG_CPU_COUNT];
 	u64 *mem_rw_tte[CONFIG_CPU_COUNT];
+	physical_addr_t mem_rw_outaddr_mask[CONFIG_CPU_COUNT];
 };
 
 static struct mmu_lpae_ctrl mmuctrl;
@@ -504,7 +505,7 @@ int mmu_lpae_unmap_page(struct cpu_ttbl *ttbl, struct cpu_page *pg)
 	cpu_mmu_sync_tte(&tte[index]);
 
 	if (ttbl->stage == TTBL_STAGE2) {
-		cpu_invalid_all_guest_tlbs();
+		cpu_invalid_ipa_guest_tlb((pg->ia));
 	} else {
 		cpu_invalid_va_hypervisor_tlb(((virtual_addr_t)pg->ia));
 	}
@@ -611,6 +612,12 @@ int mmu_lpae_map_page(struct cpu_ttbl *ttbl, struct cpu_page *pg)
 
 	cpu_mmu_sync_tte(&tte[index]);
 
+	if (ttbl->stage == TTBL_STAGE2) {
+		cpu_invalid_ipa_guest_tlb((pg->ia));
+	} else {
+		cpu_invalid_va_hypervisor_tlb(((virtual_addr_t)pg->ia));
+	}
+
 	ttbl->tte_cnt++;
 
 	vmm_spin_unlock_irqrestore_lite(&ttbl->tbl_lock, flags);
@@ -664,7 +671,7 @@ int mmu_lpae_stage2_chttbl(u8 vmid, struct cpu_ttbl *ttbl)
 
 #define PHYS_RW_TTE_NOCACHE						\
 	(PHYS_RW_TTE |							\
-	 ((AINDEX_SO << TTBL_STAGE1_LOWER_AINDEX_SHIFT) &		\
+	 ((AINDEX_NORMAL_NC << TTBL_STAGE1_LOWER_AINDEX_SHIFT) &	\
 	    TTBL_STAGE1_LOWER_AINDEX_MASK))
 
 #define PHYS_RW_TTE_CACHE						\
@@ -677,8 +684,9 @@ int arch_cpu_aspace_memory_read(virtual_addr_t tmp_va,
 				void *dst, u32 len, bool cacheable)
 {
 	u64 old_tte_val;
-	u64 *tte = mmuctrl.mem_rw_tte[vmm_smp_processor_id()];
-	struct cpu_ttbl *ttbl = mmuctrl.mem_rw_ttbl[vmm_smp_processor_id()];
+	u32 cpu = vmm_smp_processor_id();
+	u64 *tte = mmuctrl.mem_rw_tte[cpu];
+	physical_addr_t outaddr_mask = mmuctrl.mem_rw_outaddr_mask[cpu];
 	virtual_addr_t offset = (src & VMM_PAGE_MASK);
 
 	old_tte_val = *tte;
@@ -688,10 +696,10 @@ int arch_cpu_aspace_memory_read(virtual_addr_t tmp_va,
 	} else {
 		*tte = PHYS_RW_TTE_NOCACHE;
 	}
-	*tte |= src &
-		(mmu_lpae_level_map_mask(ttbl->level) & TTBL_OUTADDR_MASK);
+	*tte |= src & outaddr_mask;
 
 	cpu_mmu_sync_tte(tte);
+	cpu_invalid_va_hypervisor_tlb(tmp_va);
 
 	switch (len) {
 	case 1:
@@ -713,7 +721,6 @@ int arch_cpu_aspace_memory_read(virtual_addr_t tmp_va,
 
 	*tte = old_tte_val;
 	cpu_mmu_sync_tte(tte);
-	cpu_invalid_va_hypervisor_tlb(tmp_va);
 
 	return VMM_OK;
 }
@@ -723,8 +730,9 @@ int arch_cpu_aspace_memory_write(virtual_addr_t tmp_va,
 				 void *src, u32 len, bool cacheable)
 {
 	u64 old_tte_val;
-	u64 *tte = mmuctrl.mem_rw_tte[vmm_smp_processor_id()];
-	struct cpu_ttbl *ttbl = mmuctrl.mem_rw_ttbl[vmm_smp_processor_id()];
+	u32 cpu = vmm_smp_processor_id();
+	u64 *tte = mmuctrl.mem_rw_tte[cpu];
+	physical_addr_t outaddr_mask = mmuctrl.mem_rw_outaddr_mask[cpu];
 	virtual_addr_t offset = (dst & VMM_PAGE_MASK);
 
 	old_tte_val = *tte;
@@ -734,10 +742,10 @@ int arch_cpu_aspace_memory_write(virtual_addr_t tmp_va,
 	} else {
 		*tte = PHYS_RW_TTE_NOCACHE;
 	}
-	*tte |= dst &
-		(mmu_lpae_level_map_mask(ttbl->level) & TTBL_OUTADDR_MASK);
+	*tte |= dst & outaddr_mask;
 
 	cpu_mmu_sync_tte(tte);
+	cpu_invalid_va_hypervisor_tlb(tmp_va);
 
 	switch (len) {
 	case 1:
@@ -759,7 +767,6 @@ int arch_cpu_aspace_memory_write(virtual_addr_t tmp_va,
 
 	*tte = old_tte_val;
 	cpu_mmu_sync_tte(tte);
-	cpu_invalid_va_hypervisor_tlb(tmp_va);
 
 	return VMM_OK;
 }
@@ -825,7 +832,7 @@ int __cpuinit arch_cpu_aspace_memory_rwinit(virtual_addr_t tmp_va)
 	p.xn = 1;
 	p.ns = 1;
 	p.sh = TTBL_SH_INNER_SHAREABLE;
-	p.aindex = AINDEX_SO;
+	p.aindex = AINDEX_NORMAL_NC;
 
 	rc = mmu_lpae_map_hypervisor_page(&p);
 	if (rc) {
@@ -840,6 +847,9 @@ int __cpuinit arch_cpu_aspace_memory_rwinit(virtual_addr_t tmp_va)
 	if (rc) {
 		return rc;
 	}
+	mmuctrl.mem_rw_outaddr_mask[cpu] =
+		(mmu_lpae_level_map_mask(mmuctrl.mem_rw_ttbl[cpu]->level) &
+		 TTBL_OUTADDR_MASK);
 
 	return VMM_OK;
 }
@@ -873,15 +883,14 @@ int arch_cpu_aspace_map(virtual_addr_t page_va,
 		p.aindex = AINDEX_NORMAL_WT;
 	} else if (mem_flags & VMM_MEMORY_BUFFERABLE) {
 		p.aindex = AINDEX_NORMAL_WB;
+	} else if (mem_flags & VMM_MEMORY_IO_DEVICE) {
+		p.aindex = AINDEX_DEVICE_nGnRE;
+	} else if (mem_flags & VMM_MEMORY_DMA_COHERENT) {
+		p.aindex = AINDEX_NORMAL_WB;
+	} else if (mem_flags & VMM_MEMORY_DMA_NONCOHERENT) {
+		p.aindex = AINDEX_NORMAL_NC;
 	} else {
-		p.aindex = AINDEX_SO;
-	}
-
-	/* Force strongly-ordered non-cacheable device
-	 * memory when dma-coherent memory is required.
-	 */
-	if (mem_flags & VMM_MEMORY_DMACOHERENT) {
-		p.aindex = AINDEX_SO;
+		p.aindex = AINDEX_DEVICE_nGnRnE;
 	}
 
 	return mmu_lpae_map_hypervisor_page(&p);
@@ -914,16 +923,6 @@ int arch_cpu_aspace_va2pa(virtual_addr_t va, physical_addr_t *pa)
 	return VMM_OK;
 }
 
-int arch_cpu_aspace_pa2va(physical_addr_t pa,
-			  virtual_size_t sz,
-			  u32 mem_flags,
-			  virtual_addr_t *va)
-{
-	*va = vmm_host_memmap(pa, sz, mem_flags);
-
-	return VMM_OK;
-}
-
 int __init arch_cpu_aspace_primary_init(physical_addr_t *core_resv_pa,
 					virtual_addr_t *core_resv_va,
 					virtual_size_t *core_resv_sz,
