@@ -61,6 +61,7 @@ static void cmd_host_usage(struct vmm_chardev *cdev)
 	vmm_cprintf(cdev, "   host extirq stats\n");
 	vmm_cprintf(cdev, "   host ram info\n");
 	vmm_cprintf(cdev, "   host ram bitmap [<column count>]\n");
+	vmm_cprintf(cdev, "   host ram reserve <physaddr> <size>\n");
 	vmm_cprintf(cdev, "   host vapool info\n");
 	vmm_cprintf(cdev, "   host vapool state\n");
 	vmm_cprintf(cdev, "   host vapool bitmap [<column count>]\n");
@@ -172,12 +173,34 @@ static void cmd_host_cpu_stats(struct vmm_chardev *cdev)
 			  "-------------------------\n");
 }
 
-static void cmd_host_irq_stats(struct vmm_chardev *cdev)
+static void irq_stats_print(struct vmm_chardev *cdev, u32 irqno)
 {
-	const char *irq_name;
-	u32 num, cpu, stats, count = vmm_host_irq_count();
 	struct vmm_host_irq *irq;
 	struct vmm_host_irq_chip *chip;
+	const char *irq_name;
+	u32 cpu, stats;
+
+	irq = vmm_host_irq_get(irqno);
+	irq_name = vmm_host_irq_get_name(irq);
+	if (vmm_host_irq_is_disabled(irq) || !irq_name) {
+		return;
+	}
+	chip = vmm_host_irq_get_chip(irq);
+	if (!chip || !chip->name) {
+		return;
+	}
+	vmm_cprintf(cdev, " %-5d %-20s %-13s",
+		    irqno, irq_name, chip->name);
+	for_each_online_cpu(cpu) {
+		stats = vmm_host_irq_get_count(irq, cpu);
+		vmm_cprintf(cdev, " %-10d", stats);
+	}
+	vmm_cprintf(cdev, "\n");
+}
+
+static void cmd_host_irq_stats(struct vmm_chardev *cdev)
+{
+	u32 num, cpu, irq_count, irqext_count;
 
 	vmm_cprintf(cdev, "----------------------------------------");
 	for_each_online_cpu(cpu) {
@@ -195,24 +218,16 @@ static void cmd_host_irq_stats(struct vmm_chardev *cdev)
 		vmm_cprintf(cdev, "------------");
 	}
 	vmm_cprintf(cdev, "\n");
-	for (num = 0; num < count; num++) {
-		irq = vmm_host_irq_get(num);
-		irq_name = vmm_host_irq_get_name(irq);
-		if (vmm_host_irq_is_disabled(irq) || !irq_name) {
-			continue;
-		}
-		chip = vmm_host_irq_get_chip(irq);
-		if (!chip || !chip->name) {
-			continue;
-		}
-		vmm_cprintf(cdev, " %-5d %-20s %-13s",
-				  num, irq_name, chip->name);
-		for_each_online_cpu(cpu) {
-			stats = vmm_host_irq_get_count(irq, cpu);
-			vmm_cprintf(cdev, " %-10d", stats);
-		}
-		vmm_cprintf(cdev, "\n");
+
+	irq_count = vmm_host_irq_count();
+	for (num = 0; num < irq_count; num++) {
+		irq_stats_print(cdev, num);
 	}
+	irqext_count = vmm_host_irqext_count();
+	for (num = irq_count; num < irq_count + irqext_count; num++) {
+		irq_stats_print(cdev, num);
+	}
+
 	vmm_cprintf(cdev, "----------------------------------------");
 	for_each_online_cpu(cpu) {
 		vmm_cprintf(cdev, "------------");
@@ -250,6 +265,12 @@ static void cmd_host_ram_info(struct vmm_chardev *cdev)
 
 	vmm_cprintf(cdev, "Frame Size        : %lu (0x%08lx)\n",
 					VMM_PAGE_SIZE, VMM_PAGE_SIZE);
+	vmm_cprintf(cdev, "Color Operations  : %s\n",
+					vmm_host_ram_color_ops_name());
+	vmm_cprintf(cdev, "Color Order       : %u (0x%08x)\n",
+		vmm_host_ram_color_order(), vmm_host_ram_color_order());
+	vmm_cprintf(cdev, "Color Count       : %u (0x%08x)\n",
+		vmm_host_ram_color_count(), vmm_host_ram_color_count());
 	vmm_cprintf(cdev, "Bank Count        : %d (0x%08x)\n",
 					bank_count, bank_count);
 	vmm_cprintf(cdev, "Total Free Frames : %d (0x%08x)\n",
@@ -273,6 +294,11 @@ static void cmd_host_ram_info(struct vmm_chardev *cdev)
 	}
 }
 
+static int cmd_host_ram_reserve(struct vmm_chardev *cdev, physical_addr_t paddr, int size)
+{
+	return vmm_host_ram_reserve(paddr, size);
+}
+
 static void cmd_host_ram_bitmap(struct vmm_chardev *cdev, int colcnt)
 {
 	u32 ite, count, bn, bank_count = vmm_host_ram_bank_count();
@@ -497,7 +523,9 @@ static int cmd_host_class_device_list(struct vmm_chardev *cdev,
 
 static int cmd_host_exec(struct vmm_chardev *cdev, int argc, char **argv)
 {
-	int hirq, hcpu, colcnt;
+	int hirq, hcpu, colcnt, size;
+
+	physical_addr_t physaddr;
 
 	if (argc <= 1) {
 		goto fail;
@@ -543,6 +571,10 @@ static int cmd_host_exec(struct vmm_chardev *cdev, int argc, char **argv)
 			}
 			cmd_host_ram_bitmap(cdev, colcnt);
 			return VMM_OK;
+		} else if (strcmp(argv[2], "reserve") == 0 && 4 < argc) {
+			physaddr = strtoul(argv[3], NULL, 16);
+			size = strtoul(argv[4], NULL, 16);
+			return cmd_host_ram_reserve(cdev, physaddr, size);
 		}
 	} else if ((strcmp(argv[1], "vapool") == 0) && (2 < argc)) {
 		if (strcmp(argv[2], "info") == 0) {
