@@ -24,6 +24,7 @@
 #ifndef __VMM_DEVDRV_H_
 #define __VMM_DEVDRV_H_
 
+#include <vmm_error.h>
 #include <vmm_limits.h>
 #include <vmm_types.h>
 #include <vmm_devtree.h>
@@ -31,7 +32,6 @@
 #include <vmm_mutex.h>
 #include <vmm_notifier.h>
 #include <libs/list.h>
-#include <vmm_error.h>
 
 #define VMM_DMA_BIT_MASK(n) (((n) == 64) ? ~0ULL : ((1ULL<<(n))-1))
 
@@ -42,6 +42,7 @@ struct vmm_device;
 struct vmm_driver;
 struct vmm_iommu_ops;
 struct vmm_iommu_group;
+struct vmm_msi_domain;
 
 struct vmm_class {
 	/* Private fields (for device driver framework) */
@@ -86,6 +87,8 @@ struct vmm_device {
 	vmm_spinlock_t devres_lock;
 	struct dlist devres_head;
 	struct dlist deferred_head;
+	struct dlist msi_list;
+	struct vmm_msi_domain *msi_domain;
 	/* Public fields */
 	char name[VMM_FIELD_NAME_SIZE];
 	bool autoprobe_disabled;
@@ -96,6 +99,7 @@ struct vmm_device {
 	struct vmm_class *class;
 	struct vmm_driver *driver;
 	struct vmm_iommu_group *iommu_group;
+	void *iommu_priv;
 	u64 *dma_mask;
 	void *pins;
 	void (*release) (struct vmm_device *);
@@ -115,13 +119,13 @@ struct vmm_driver {
 	int (*remove) (struct vmm_device *);
 };
 
-/** Set driver data in device */
+/** Get driver data from device */
 static inline void *vmm_devdrv_get_data(const struct vmm_device *dev)
 {
 	return (dev) ? dev->priv : NULL;
 }
 
-/** Get driver data from device */
+/** Set driver data in device */
 static inline void vmm_devdrv_set_data(struct vmm_device *dev, void *data)
 {
 	if (dev) {
@@ -129,6 +133,22 @@ static inline void vmm_devdrv_set_data(struct vmm_device *dev, void *data)
 	}
 }
 
+/** Get MSI domain from device */
+static inline struct vmm_msi_domain *vmm_devdrv_get_msi_domain(
+						struct vmm_device *dev)
+{
+	return (dev) ? dev->msi_domain : NULL;
+}
+
+/** Set MSI domain in device */
+static inline void vmm_devdrv_set_msi_domain(struct vmm_device *dev,
+					     struct vmm_msi_domain *domain)
+{
+	if (dev) {
+		dev->msi_domain = domain;
+	}
+}
+
 /** get the dma_mask from device */
 static inline u64 vmm_dma_get_mask(struct vmm_device *dev)
 {
@@ -157,9 +177,6 @@ static inline int vmm_dma_set_mask(struct vmm_device *dev, u64 mask)
  */
 int vmm_devdrv_pinctrl_bind(struct vmm_device *dev);
 
-/** Probe device instances under a given device tree node */
-int vmm_devdrv_probe(struct vmm_devtree_node *node);
-
 /** Register class */
 int vmm_devdrv_register_class(struct vmm_class *cls);
 
@@ -221,6 +238,11 @@ struct vmm_device *vmm_devdrv_bus_find_device_by_name(struct vmm_bus *bus,
 						struct vmm_device *start,
 						const char *dname);
 
+/** Find device on a bus by node */
+struct vmm_device *vmm_devdrv_bus_find_device_by_node(struct vmm_bus *bus,
+						struct vmm_device *start,
+						struct vmm_devtree_node *np);
+
 /** Iterate over each device of a bus with bus->lock held */
 int vmm_devdrv_bus_device_iterate(struct vmm_bus *bus,
 			struct vmm_device *start, void *data,
