@@ -42,20 +42,29 @@
 #define VMM_DEVTREE_DEVICE_TYPE_VAL_VCPU	"vcpu"
 #define VMM_DEVTREE_DEVICE_TYPE_VAL_RAM		"ram"
 #define VMM_DEVTREE_DEVICE_TYPE_VAL_ALLOCED_RAM	"alloced_ram"
+#define VMM_DEVTREE_DEVICE_TYPE_VAL_COLORED_RAM	"colored_ram"
+#define VMM_DEVTREE_DEVICE_TYPE_VAL_SHARED_RAM	"shared_ram"
 #define VMM_DEVTREE_DEVICE_TYPE_VAL_ROM		"rom"
 #define VMM_DEVTREE_DEVICE_TYPE_VAL_ALLOCED_ROM	"alloced_rom"
+#define VMM_DEVTREE_DEVICE_TYPE_VAL_COLORED_ROM	"colored_rom"
+#define VMM_DEVTREE_DEVICE_TYPE_VAL_SHARED_ROM	"shared_rom"
 #define VMM_DEVTREE_COMPATIBLE_ATTR_NAME	"compatible"
 #define VMM_DEVTREE_CLOCK_FREQ_ATTR_NAME	"clock-frequency"
 #define VMM_DEVTREE_CLOCKS_ATTR_NAME		"clocks"
 #define VMM_DEVTREE_CLOCK_NAMES_ATTR_NAME	"clock-names"
 #define VMM_DEVTREE_CLOCK_OUT_NAMES_ATTR_NAME	"clock-output-names"
 #define VMM_DEVTREE_REG_ATTR_NAME		"reg"
+#define VMM_DEVTREE_REG_NAMES_ATTR_NAME		"reg-names"
 #define VMM_DEVTREE_VIRTUAL_REG_ATTR_NAME	"virtual-reg"
 #define VMM_DEVTREE_RANGES_ATTR_NAME		"ranges"
+#define VMM_DEVTREE_BIG_ENDIAN_ATTR_NAME	"big-endian"
+#define VMM_DEVTREE_NATIVE_ENDIAN_ATTR_NAME	"native-endian"
 #define VMM_DEVTREE_ADDR_CELLS_ATTR_NAME	"#address-cells"
 #define VMM_DEVTREE_SIZE_CELLS_ATTR_NAME	"#size-cells"
 #define VMM_DEVTREE_PHANDLE_ATTR_NAME		"phandle"
 
+#define VMM_DEVTREE_DEBUG_ATTR_NAME		"debug"
+
 #define VMM_DEVTREE_CHOSEN_NODE_NAME		"chosen"
 #define VMM_DEVTREE_CONSOLE_ATTR_NAME		"console"
 #define VMM_DEVTREE_RTCDEV_ATTR_NAME		"rtcdev"
@@ -102,10 +111,19 @@
 #define VMM_DEVTREE_ALIAS_PHYS_ATTR_NAME	"alias_physical_addr"
 #define VMM_DEVTREE_PHYS_SIZE_ATTR_NAME		"physical_size"
 #define VMM_DEVTREE_ALIGN_ORDER_ATTR_NAME	"align_order"
+#define VMM_DEVTREE_FIRST_COLOR_ATTR_NAME	"first_color"
+#define VMM_DEVTREE_NUM_COLORS_ATTR_NAME	"num_colors"
+#define VMM_DEVTREE_SHARED_MEM_ATTR_NAME	"shared_mem"
+#define VMM_DEVTREE_MAP_ORDER_ATTR_NAME		"map_order"
 #define VMM_DEVTREE_SWITCH_ATTR_NAME		"switch"
+#define VMM_DEVTREE_DOMAIN_ATTR_NAME		"domain"
+#define VMM_DEVTREE_NODE_ADDR_ATTR_NAME		"node_addr"
+#define VMM_DEVTREE_NODE_NS_NAME_ATTR_NAME	"node_ns_name"
 #define VMM_DEVTREE_BLKDEV_ATTR_NAME		"blkdev"
 #define VMM_DEVTREE_VCPU_AFFINITY_ATTR_NAME	"affinity"
 #define VMM_DEVTREE_VCPU_POWEROFF_ATTR_NAME	"poweroff"
+#define VMM_DEVTREE_NO_CHILD_PROBE_ATTR_NAME	"no-child-probe"
+#define VMM_DEVTREE_THREADS_AFFINITY_ATTR_NAME	"threads_affinity"
 
 enum vmm_devtree_attrypes {
 	VMM_DEVTREE_ATTRTYPE_UINT32	= 0,
@@ -411,6 +429,18 @@ const u32 *vmm_devtree_next_u32(struct vmm_devtree_attr *attr,
 const char *vmm_devtree_next_string(struct vmm_devtree_attr *attr,
 				    const char *cur);
 
+#define vmm_devtree_for_each_u32(np, attrname, attr, p, u)	\
+	for (attr = vmm_devtree_getattr(np, attrname),		\
+		p = vmm_devtree_next_u32(attr, NULL, &u);	\
+		p;						\
+		p = vmm_devtree_next_u32(attr, p, &u))
+
+#define vmm_devtree_for_each_string(np, attrname, attr, s)      \
+        for (attr = vmm_devtree_getattr(np, attrname),		\
+                s = vmm_devtree_next_string(attr, NULL);	\
+                s;						\
+                s = vmm_devtree_next_string(attr, s))
+
 /** Create a path string for a given node */
 int vmm_devtree_getpath(char *out, size_t out_len,
 			const struct vmm_devtree_node *node);
@@ -618,12 +648,6 @@ int vmm_devtree_delnode(struct vmm_devtree_node *node);
 int vmm_devtree_clock_frequency(struct vmm_devtree_node *node,
 				u32 *clock_freq);
 
-/** Get device irq number
- *  NOTE: This is based on 'irq' attribute of device tree node
- */
-int vmm_devtree_irq_get(struct vmm_devtree_node *node,
-		        u32 *irq, int index);
-
 /** Get count of device irqs
  *  NOTE: This is based on 'irq' attribute of device tree node
  */
@@ -663,10 +687,10 @@ unsigned int vmm_devtree_irq_parse_map(struct vmm_devtree_node *dev,
 /** vmm_devtree_is_available - check if a device is available for use
  *  @node: Node to check for availability
  *
- *  Returns 1 if the status property is absent or set to "okay" or "ok",
- *  0 otherwise
+ *  Returns TRUE if the status property is absent or set to "okay" or "ok",
+ *  FALSE otherwise
  */
-u32 vmm_devtree_is_available(struct vmm_devtree_node *node);
+bool vmm_devtree_is_available(const struct vmm_devtree_node *node);
 
 /** vmm_devtree_alias_get_id - Get alias id for the given device_node
  * @np:         Pointer to the given device_node
@@ -706,6 +730,26 @@ int vmm_devtree_regmap(struct vmm_devtree_node *node,
 int vmm_devtree_regunmap(struct vmm_devtree_node *node,
 			 virtual_addr_t addr, int regset);
 
+/** Convert regname to regset index
+ *  NOTE: This is based on 'reg-names' attribute of device tree node
+ */
+int vmm_devtree_regname_to_regset(struct vmm_devtree_node *node,
+				  const char *regname);
+
+/** Map device registers to virtual address
+ *  NOTE: This is based on 'reg' and 'reg-names' attributes
+ *  of device tree node
+ */
+int vmm_devtree_regmap_byname(struct vmm_devtree_node *node,
+			      virtual_addr_t *addr, const char *regname);
+
+/** Unmap device registers from virtual address
+ *  NOTE: This is based on 'reg' and 'reg-names' attributes
+ *  of device tree node
+ */
+int vmm_devtree_regunmap_byname(struct vmm_devtree_node *node,
+				virtual_addr_t addr, const char *regname);
+
 /** Request hostmem resource region for device registers physical
  *  address and Map device registers to a virtual address
  *  NOTE: This is based on 'reg' attribute of device tree node
@@ -721,6 +765,9 @@ int vmm_devtree_request_regmap(struct vmm_devtree_node *node,
 int vmm_devtree_regunmap_release(struct vmm_devtree_node *node,
 				 virtual_addr_t addr, int regset);
 
+/** Check whether device registers are big endian */
+bool vmm_devtree_is_reg_big_endian(struct vmm_devtree_node *node);
+
 /** Count number of enteries in nodeid table */
 u32 vmm_devtree_nidtbl_count(void);
 
