@@ -54,14 +54,12 @@ enum vmm_irq_trigger_types {
 /**
  * enum vmm_irq_states
  * @VMM_IRQ_STATE_TRIGGER_MASK		- Mask for the trigger type bits
- * @VMM_IRQ_STATE_NO_BALANCING		- Balancing disabled for this IRQ
  * @VMM_IRQ_STATE_PER_CPU		- Interrupt is per cpu
  * @VMM_IRQ_STATE_AFFINITY_SET		- Interrupt affinity was set
  * @VMM_IRQ_STATE_LEVEL			- Interrupt is level triggered
  * @VMM_IRQ_STATE_ROUTED		- Interrupt is routed to some guest
  * @VMM_IRQ_STATE_IPI			- Interrupt is an inter-processor interrupt
- * @VMM_IRQ_STATE_DISABLED		- Disabled state of the interrupt
- * @VMM_IRQ_STATE_MASKED		- Masked state of the interrupt
+ * @VMM_IRQ_STATE_EXTENDED		- Interrupt is an extended interrupt
  */
 enum vmm_irq_states {
 	VMM_IRQ_STATE_TRIGGER_MASK	= 0xf,
@@ -70,8 +68,17 @@ enum vmm_irq_states {
 	VMM_IRQ_STATE_LEVEL		= (1 << 13),
 	VMM_IRQ_STATE_ROUTED		= (1 << 14),
 	VMM_IRQ_STATE_IPI		= (1 << 15),
-	VMM_IRQ_STATE_DISABLED		= (1 << 16),
-	VMM_IRQ_STATE_MASKED		= (1 << 17),
+	VMM_IRQ_STATE_EXTENDED		= (1 << 16),
+};
+
+/**
+ * enum vmm_percpu_irq_states
+ * @VMM_PERCPU_IRQ_STATE_IN_PROG	- Interrupt in-progress
+ * @VMM_PERCPU_IRQ_STATE_MASKED		- Interrupt masked
+ */
+enum vmm_percpu_irq_states {
+	VMM_PERCPU_IRQ_STATE_IN_PROG	= (1 << 0),
+	VMM_PERCPU_IRQ_STATE_MASKED	= (1 << 1),
 };
 
 /**
@@ -147,8 +154,8 @@ struct vmm_host_irq {
 	u32 hwirq;
 	const char *name;
 	u32 state;
+	u32 percpu_state[CONFIG_CPU_COUNT];
 	u32 count[CONFIG_CPU_COUNT];
-	bool in_progress[CONFIG_CPU_COUNT];
 	void *chip_data;
 	struct vmm_host_irq_chip *chip;
 	vmm_host_irq_handler_t handler;
@@ -182,13 +189,17 @@ void vmm_host_irq_set_active_callback(u32 (*active)(u32));
  *  Note: This function is for internal use only.
  *  Note: Do not call this function directly.
  */
-void __vmm_host_irq_init_desc(struct vmm_host_irq *irq, u32 num, u32 hwirq);
+void __vmm_host_irq_init_desc(struct vmm_host_irq *irq,
+			      u32 hirq, u32 hwirq, u32 state);
 
 /** Get host irq count */
 u32 vmm_host_irq_count(void);
 
-/** Set hwirq associated with host irq instance */
-int vmm_host_irq_set_hwirq(u32 hirq, u32 hwirq);
+/** Set hwirq associated with host irq instance
+ *  Note: This function is for internal use only.
+ *  Note: Do not call this function directly.
+ */
+int __vmm_host_irq_set_hwirq(u32 hirq, u32 hwirq);
 
 /** Get hwirq associated with host irq instance */
 u32 vmm_host_irq_get_hwirq(u32 hirq);
@@ -300,23 +311,21 @@ static inline bool vmm_host_irq_is_ipi(struct vmm_host_irq *irq)
 	return (irq->state & VMM_IRQ_STATE_IPI) ? TRUE : FALSE;
 }
 
+/** Check if a host irq is masked */
+bool vmm_host_irq_is_masked(struct vmm_host_irq *irq);
+
 /** Check if a host irq is disabled */
 static inline bool vmm_host_irq_is_disabled(struct vmm_host_irq *irq)
 {
-	return (irq->state & VMM_IRQ_STATE_DISABLED) ? TRUE : FALSE;
-}
-
-/** Check if a host irq is masked */
-static inline bool vmm_host_irq_is_masked(struct vmm_host_irq *irq)
-{
-	return (irq->state & VMM_IRQ_STATE_MASKED) ? TRUE : FALSE;
+	return vmm_host_irq_is_masked(irq);
 }
 
 /** Check if a host irq is in-progress */
 static inline bool vmm_host_irq_is_inprogress(struct vmm_host_irq *irq, u32 cpu)
 {
-	if (cpu < CONFIG_CPU_COUNT) {
-		return (irq) ? irq->in_progress[cpu] : FALSE;
+	if (irq && (cpu < CONFIG_CPU_COUNT)) {
+		return (irq->percpu_state[cpu] & VMM_PERCPU_IRQ_STATE_IN_PROG)
+			? TRUE : FALSE;
 	}
 	return FALSE;
 }
@@ -362,18 +371,24 @@ int vmm_host_irq_mark_ipi(u32 hirq);
 /** UnMark host irq as inter-processor interrupt */
 int vmm_host_irq_unmark_ipi(u32 hirq);
 
-/** Enable a host irq (by default all irqs are disabled) */
-int vmm_host_irq_enable(u32 hirq);
-
-/** Disable a host irq */
-int vmm_host_irq_disable(u32 hirq);
-
 /** Unmask a host irq (by default all irqs are masked) */
 int vmm_host_irq_unmask(u32 hirq);
 
 /** Mask a host irq */
 int vmm_host_irq_mask(u32 hirq);
 
+/** Enable a host irq  */
+static inline int vmm_host_irq_enable(u32 hirq)
+{
+	return vmm_host_irq_unmask(hirq);
+}
+
+/** Disable a host irq */
+static inline int vmm_host_irq_disable(u32 hirq)
+{
+	return vmm_host_irq_mask(hirq);
+}
+
 /** Raise a host irq from software */
 int vmm_host_irq_raise(u32 hirq,
 		       const struct vmm_cpumask *dest);
