@@ -29,7 +29,7 @@
 
 #define BUG_ON(x)							\
 	do {								\
-		if (x) {						\
+		if (unlikely(x)) {					\
 			vmm_lemergency(NULL, "Bug in %s() at %s:%d\n",	\
 				   __func__, __FILE__, __LINE__);	\
 			dump_stacktrace();				\
@@ -41,7 +41,7 @@
 
 #define WARN_ON(x)							\
 	({								\
-		if (x) {						\
+		if (unlikely(x)) {					\
 			vmm_lwarning(NULL, "%s() at %s:%d\n",		\
 				   __func__, __FILE__, __LINE__);	\
 			dump_stacktrace();				\
@@ -51,7 +51,7 @@
 
 #define WARN(x, msg...)							\
 	({								\
-		if (x) {						\
+		if (unlikely(x)) {					\
 			vmm_lwarning(NULL, msg);			\
 			vmm_lwarning(NULL, "%s() at %s:%d\n",		\
 				   __func__, __FILE__, __LINE__);	\
@@ -127,6 +127,10 @@ int __printf(2, 3) vmm_cprintf(struct vmm_chardev *cdev,
 /** Print formatted string to default device */
 #define vmm_printf(args...)	vmm_cprintf(NULL, args)
 
+/** Print contents of some data in hex format */
+void vmm_chexdump(struct vmm_chardev *cdev,
+		  u64 print_base_addr, void *data, u64 len);
+
 /** Predefined log levels */
 enum vmm_print_level {
 	VMM_LOGLEVEL_EMERGENCY=0,
