@@ -24,6 +24,7 @@
 #include <vmm_error.h>
 #include <vmm_stdio.h>
 #include <vmm_completion.h>
+#include <arch_cpu_irq.h>
 
 bool vmm_completion_done(struct vmm_completion *cmpl)
 {
@@ -46,11 +47,12 @@ bool vmm_completion_done(struct vmm_completion *cmpl)
 static int completion_wait_common(struct vmm_completion *cmpl, u64 *timeout)
 {
 	int rc = VMM_OK;
+	irq_flags_t flags;
 
 	BUG_ON(!cmpl);
 	BUG_ON(!vmm_scheduler_orphan_context());
 
-	vmm_spin_lock_irq(&cmpl->wq.lock);
+	vmm_spin_lock_irqsave(&cmpl->wq.lock, flags);
 
 	if (!cmpl->done) {
 		rc = __vmm_waitqueue_sleep(&cmpl->wq, timeout);
@@ -59,7 +61,7 @@ static int completion_wait_common(struct vmm_completion *cmpl, u64 *timeout)
 		cmpl->done--;
 	}
 
-	vmm_spin_unlock_irq(&cmpl->wq.lock);
+	vmm_spin_unlock_irqrestore(&cmpl->wq.lock, flags);
 
 	return rc;
 }
@@ -91,6 +93,25 @@ int vmm_completion_complete(struct vmm_completion *cmpl)
 	return rc;
 }
 
+int vmm_completion_complete_once(struct vmm_completion *cmpl)
+{
+	int rc = VMM_OK;
+	irq_flags_t flags;
+
+	BUG_ON(!cmpl);
+
+	vmm_spin_lock_irqsave(&cmpl->wq.lock, flags);
+
+	if (!cmpl->done) {
+		cmpl->done++;
+		rc = __vmm_waitqueue_wakefirst(&cmpl->wq);
+	}
+
+	vmm_spin_unlock_irqrestore(&cmpl->wq.lock, flags);
+
+	return rc;
+}
+
 int vmm_completion_complete_all(struct vmm_completion *cmpl)
 {
 	int rc = VMM_OK;
