@@ -26,6 +26,7 @@
 #include <vmm_devtree.h>
 #include <vmm_stdio.h>
 #include <vmm_version.h>
+#include <vmm_initfn.h>
 #include <vmm_host_aspace.h>
 #include <vmm_host_irq.h>
 #include <vmm_smp.h>
@@ -34,6 +35,7 @@
 #include <vmm_clockchip.h>
 #include <vmm_timer.h>
 #include <vmm_delay.h>
+#include <vmm_shmem.h>
 #include <vmm_manager.h>
 #include <vmm_scheduler.h>
 #include <vmm_loadbal.h>
@@ -45,6 +47,7 @@
 #include <vmm_cmdmgr.h>
 #include <vmm_wallclock.h>
 #include <vmm_chardev.h>
+#include <vmm_iommu.h>
 #include <vmm_modules.h>
 #include <vmm_extable.h>
 #include <arch_cpu.h>
@@ -180,20 +183,8 @@ static void system_init_work(struct vmm_work *work)
 	}
 
 #if defined(CONFIG_SMP)
-	/* Initialize secondary CPUs */
-	vmm_printf("init: secondary CPUs\n");
-	ret = arch_smp_init_cpus();
-	if (ret) {
-		goto fail;
-	}
-
-	/* Prepare secondary CPUs */
-	ret = arch_smp_prepare_cpus(vmm_num_possible_cpus());
-	if (ret) {
-		goto fail;
-	}
-
 	/* Start each present secondary CPUs */
+	vmm_printf("init: start secondary CPUs\n");
 	for_each_present_cpu(c) {
 		if (c == vmm_smp_bootcpu_id()) {
 			continue;
@@ -205,12 +196,14 @@ static void system_init_work(struct vmm_work *work)
 		}
 	}
 
+#ifdef CONFIG_LOADBAL
 	/* Initialize hypervisor load balancer */
 	vmm_printf("init: hypervisor load balancer\n");
 	ret = vmm_loadbal_init();
 	if (ret) {
 		goto fail;
 	}
+#endif
 #endif
 
 	/* Initialize command manager */
@@ -244,8 +237,8 @@ static void system_init_work(struct vmm_work *work)
 #if defined(CONFIG_SMP)
 	/* Poll for all present CPUs to become online */
 	/* Note: There is a timeout of 1 second */
-	/* Note: The modules might use SMP IPIs or might have per-cpu context 
-	 * so, we do this before vmm_modules_init() in-order to make sure that 
+	/* Note: The modules might use SMP IPIs or might have per-cpu context
+	 * so, we do this before vmm_modules_init() in-order to make sure that
 	 * correct number of online CPUs are visible to all modules.
 	 */
 	ret = 1000;
@@ -266,6 +259,13 @@ static void system_init_work(struct vmm_work *work)
 	}
 #endif
 
+	/* Initialize IOMMU framework */
+	vmm_printf("init: iommu framework\n");
+	ret = vmm_iommu_init();
+	if (ret) {
+		goto fail;
+	}
+
 	/* Initialize hypervisor modules */
 	vmm_printf("init: hypervisor modules\n");
 	ret = vmm_modules_init();
@@ -287,6 +287,13 @@ static void system_init_work(struct vmm_work *work)
 		goto fail;
 	}
 
+	/* Call final init functions */
+	vmm_printf("init: final functions\n");
+	ret = vmm_initfn_final();
+	if (ret) {
+		goto fail;
+	}
+
 	/* Schedule system post-init work */
 	INIT_WORK(&sys_postinit, &system_postinit_work);
 	vmm_workqueue_schedule_work(NULL, &sys_postinit);
@@ -313,7 +320,7 @@ static void __init init_bootcpu(void)
 
 	/* Print version string */
 	vmm_printf("\n");
-	vmm_printf("%s v%d.%d.%d (%s %s)\n", VMM_NAME, 
+	vmm_printf("%s v%d.%d.%d (%s %s)\n", VMM_NAME,
 		   VMM_VERSION_MAJOR, VMM_VERSION_MINOR, VMM_VERSION_RELEASE,
 		   __DATE__, __TIME__);
 	vmm_printf("\n");
@@ -339,16 +346,31 @@ static void __init init_bootcpu(void)
 		goto init_bootcpu_fail;
 	}
 
-	/* Initialize per-cpu area */
-	vmm_printf("init: per-CPU areas\n");
-	ret = vmm_percpu_init();
+	/* Initialize device tree */
+	vmm_printf("init: device tree\n");
+	ret = vmm_devtree_init();
 	if (ret) {
 		goto init_bootcpu_fail;
 	}
 
-	/* Initialize device tree */
-	vmm_printf("init: device tree\n");
-	ret = vmm_devtree_init();
+#if defined(CONFIG_SMP)
+	/* Initialize secondary CPUs */
+	vmm_printf("init: discover secondary CPUs\n");
+	ret = arch_smp_init_cpus();
+	if (ret) {
+		goto init_bootcpu_fail;
+	}
+
+	/* Prepare secondary CPUs */
+	ret = arch_smp_prepare_cpus(vmm_num_possible_cpus());
+	if (ret) {
+		goto init_bootcpu_fail;
+	}
+#endif
+
+	/* Initialize per-cpu area */
+	vmm_printf("init: per-CPU areas\n");
+	ret = vmm_percpu_init();
 	if (ret) {
 		goto init_bootcpu_fail;
 	}
@@ -390,6 +412,13 @@ static void __init init_bootcpu(void)
 		goto init_bootcpu_fail;
 	}
 
+	/* Call early init functions */
+	vmm_printf("init: early funtions\n");
+	ret = vmm_initfn_early();
+	if (ret) {
+		goto init_bootcpu_fail;
+	}
+
 	/* Initialize standerd input/output */
 	vmm_printf("init: standard I/O\n");
 	ret = vmm_stdio_init();
@@ -418,13 +447,20 @@ static void __init init_bootcpu(void)
 		goto init_bootcpu_fail;
 	}
 
-	/* Initialize soft delay */
-	vmm_printf("init: soft delay\n");
+	/* Initialize hypervisor soft delay */
+	vmm_printf("init: hypervisor soft delay\n");
 	ret = vmm_delay_init();
 	if (ret) {
 		goto init_bootcpu_fail;
 	}
 
+	/* Initialize hypervisor shared memory */
+	vmm_printf("init: hypervisor shared memory\n");
+	ret = vmm_shmem_init();
+	if (ret) {
+		goto init_bootcpu_fail;
+	}
+
 	/* Initialize hypervisor manager */
 	vmm_printf("init: hypervisor manager\n");
 	ret = vmm_manager_init();
