@@ -88,7 +88,7 @@ struct module_wrap {
 struct vmm_modules_ctrl {
 	vmm_spinlock_t lock;
 	struct dlist mod_list;
-        u32 mod_count;
+	u32 mod_count;
 };
 
 static struct vmm_modules_ctrl modctrl;
@@ -98,7 +98,6 @@ int vmm_modules_find_symbol(const char *symname, struct vmm_symbol *sym)
 	u32 s;
 	bool found;
 	irq_flags_t flags;
-	struct dlist *l;
 	struct module_wrap *mwrap = NULL;
 
 	if (!symname || !sym) {
@@ -107,7 +106,10 @@ int vmm_modules_find_symbol(const char *symname, struct vmm_symbol *sym)
 
 	sym->addr = kallsyms_lookup_name(symname);
 	if (sym->addr) {
-		strncpy(sym->name, symname, KSYM_NAME_LEN);
+		if (strlcpy(sym->name, symname, sizeof(sym->name)) >=
+		    sizeof(sym->name)) {
+			return VMM_EOVERFLOW;
+		}
 		sym->type = VMM_SYMBOL_GPL;
 		return VMM_OK;
 	}
@@ -115,8 +117,7 @@ int vmm_modules_find_symbol(const char *symname, struct vmm_symbol *sym)
 	vmm_spin_lock_irqsave(&modctrl.lock, flags);
 
 	found = FALSE;
-	list_for_each(l, &modctrl.mod_list) {
-		mwrap = list_entry(l, struct module_wrap, head);
+	list_for_each_entry(mwrap, &modctrl.mod_list, head) {
 		for (s = 0; s < mwrap->num_syms; s++) {
 			if (strcmp(mwrap->syms[s].name, symname) == 0) {
 				memcpy(sym, &mwrap->syms[s], sizeof(*sym));
@@ -154,6 +155,8 @@ bool vmm_modules_isbuiltin(struct vmm_module *mod)
 	return (mwrap->built_in) ? TRUE : FALSE;
 }
 
+#ifdef CONFIG_MODULES
+
 /* Find a module section: 0 means not found. */
 static u32 find_sec(const struct load_info *info, const char *name)
 {
@@ -256,6 +259,11 @@ static int setup_load_info(struct load_info *info)
 		}
 	}
 
+	/* If symbol table not found then return failure */
+	if (!info->strtab) {
+		return VMM_ENOTAVAIL;
+	}
+
 	return VMM_OK;
 }
 
@@ -437,10 +445,15 @@ static int move_module(struct module_wrap *mwrap,
 			struct load_info *info)
 {
 	u32 i;
+	virtual_addr_t addr =
+		vmm_host_alloc_pages(VMM_SIZE_TO_PAGE(mwrap->core_size),
+				     VMM_MEMORY_FLAGS_NORMAL);
+	if (!addr) {
+		return VMM_ENOMEM;
+	}
 
 	mwrap->pg_count = VMM_SIZE_TO_PAGE(mwrap->core_size);
-	mwrap->pg_start = vmm_host_alloc_pages(mwrap->pg_count, 
-						VMM_MEMORY_FLAGS_NORMAL);
+	mwrap->pg_start = addr;
 
 	memset((void *)mwrap->pg_start, 0, mwrap->core_size);
 
@@ -703,27 +716,44 @@ int vmm_modules_unload(struct vmm_module *mod)
 	return VMM_OK;
 }
 
+#else
+
+int vmm_modules_load(virtual_addr_t load_addr, virtual_size_t load_size)
+{
+	return VMM_ENOTAVAIL;
+}
+
+int vmm_modules_unload(struct vmm_module *mod)
+{
+	return VMM_ENOTAVAIL;
+}
+
+#endif
+
 struct vmm_module *vmm_modules_getmodule(u32 index)
 {
+	bool found = FALSE;
 	irq_flags_t flags;
-	struct dlist *l;
-	struct module_wrap *ret = NULL;
+	struct module_wrap *mwrap;
 
 	vmm_spin_lock_irqsave(&modctrl.lock, flags);
 
-	if (0 <= index && index < modctrl.mod_count) {
-		list_for_each(l, &modctrl.mod_list) {
-			ret = list_entry(l, struct module_wrap, head);
-			if (!index) {
-				break;
-			}
-			index--;
+	if (modctrl.mod_count <= index) {
+		vmm_spin_unlock_irqrestore(&modctrl.lock, flags);
+		return NULL;
+	}
+
+	list_for_each_entry(mwrap, &modctrl.mod_list, head) {
+		if (!index) {
+			found = TRUE;
+			break;
 		}
+		index--;
 	}
 
 	vmm_spin_unlock_irqrestore(&modctrl.lock, flags);
 
-	return (ret) ? &ret->mod : NULL;
+	return (found) ? &mwrap->mod : NULL;
 }
 
 u32 vmm_modules_count(void)
@@ -752,6 +782,7 @@ static struct modules_list * __init aggregate_modules(u32 mod_start, u32 sz)
 		(struct modules_list *)vmm_malloc(sizeof(struct modules_list));
 
 	INIT_LIST_HEAD(&cong_mod_list->mod_list);
+	cong_mod_list->nr_modules = 0;
 	/*
 	 * Search for modules signatures from mod start to
 	 * end of modules area picking up modules information
@@ -789,7 +820,6 @@ int __init vmm_modules_init(void)
 	struct module_wrap *mwrap;
 	struct vmm_module *mod_entry;
 	struct modules_list *ag_mod_list;
-	struct dlist *tnode;
 
 	/* Reset the control structure */
 	memset(&modctrl, 0, sizeof(modctrl));
@@ -808,11 +838,7 @@ int __init vmm_modules_init(void)
 	list_mergesort(NULL, &ag_mod_list->mod_list, cmp_list_element);
 
 	/* Initialize built-in modules in sorted order */
-	list_for_each(tnode, &ag_mod_list->mod_list) {
-		mod_entry = list_entry(tnode, struct vmm_module, head);
-		if (unlikely(!mod_entry))
-			break;
-
+	list_for_each_entry(mod_entry, &ag_mod_list->mod_list, head) {
 		mwrap = vmm_zalloc(sizeof(struct module_wrap));
 		if (unlikely(!mwrap)) {
 			break;
@@ -825,7 +851,7 @@ int __init vmm_modules_init(void)
 		/* Initialize module if required */
 		if (mwrap->mod.init) {
 #if defined(CONFIG_VERBOSE_MODE)
-			vmm_printf("Initialize %s\n", mwrap->mod.name);
+			vmm_printf("Module Init %s\n", mwrap->mod.name);
 #endif
 			if ((ret = mwrap->mod.init())) {
 				vmm_printf("%s: %s init error %d\n", 
