@@ -40,6 +40,60 @@
 /* the following should be enough for 32 bit int */
 #define PRINT_BUF_LEN	64
 
+#ifdef CONFIG_LOG_ANSI_COLORS
+# define VMM_LOG_COLOR_RESET		"\033[0m"
+# define VMM_LOG_COLOR_LIGHTRED		"\033[31;1m"
+# define VMM_LOG_COLOR_RED		"\033[31m"
+# define VMM_LOG_COLOR_LIGHTBLUE	"\033[34;1m"
+# define VMM_LOG_COLOR_BLUE		"\033[34m"
+# define VMM_LOG_COLOR_GREEN		"\033[32;1m"
+# define VMM_LOG_COLOR_YELLOW		"\033[33;1m"
+# define VMM_LOG_COLOR_ORANGE		"\033[0;33m"
+# define VMM_LOG_COLOR_WHITE		"\033[37;1m"
+# define VMM_LOG_COLOR_LIGHTCYAN	"\033[36;1m"
+# define VMM_LOG_COLOR_CYAN		"\033[36m"
+# define VMM_LOG_COLOR_REVERSE		"\033[7m"
+#else /* ! CONFIG_LOG_ANSI_COLORS */
+# define VMM_LOG_COLOR_RESET		""
+# define VMM_LOG_COLOR_LIGHTRED		""
+# define VMM_LOG_COLOR_RED		""
+# define VMM_LOG_COLOR_LIGHTBLUE	""
+# define VMM_LOG_COLOR_BLUE		""
+# define VMM_LOG_COLOR_GREEN		""
+# define VMM_LOG_COLOR_YELLOW		""
+# define VMM_LOG_COLOR_ORANGE		""
+# define VMM_LOG_COLOR_WHITE		""
+# define VMM_LOG_COLOR_LIGHTCYAN	""
+# define VMM_LOG_COLOR_CYAN		""
+# define VMM_LOG_COLOR_REVERSE		""
+#endif /* CONFIG_LOG_ANSI_COLORS */
+
+/* Strings used when prefixing a log level */
+#define VMM_LOG_INFO			\
+	VMM_LOG_COLOR_CYAN     VMM_LOG_COLOR_REVERSE "INFO:"
+#define VMM_LOG_NOTICE			\
+	VMM_LOG_COLOR_GREEN    VMM_LOG_COLOR_REVERSE "NOTE:"
+#define VMM_LOG_WARNING			\
+	VMM_LOG_COLOR_YELLOW   VMM_LOG_COLOR_REVERSE "WARNING:"
+#define VMM_LOG_ERROR			\
+	VMM_LOG_COLOR_ORANGE   VMM_LOG_COLOR_REVERSE "ERROR:"
+#define VMM_LOG_CRITICAL		\
+	VMM_LOG_COLOR_RED      VMM_LOG_COLOR_REVERSE "CRITICAL:"
+#define VMM_LOG_ALERT			\
+	VMM_LOG_COLOR_LIGHTRED VMM_LOG_COLOR_REVERSE "ALERT:"
+#define VMM_LOG_EMERGENCY		\
+	VMM_LOG_COLOR_LIGHTRED VMM_LOG_COLOR_REVERSE "FATAL:"
+
+static char const* const _log_prefixes[] = {
+	[VMM_LOGLEVEL_EMERGENCY] = VMM_LOG_EMERGENCY,
+	[VMM_LOGLEVEL_ALERT]     = VMM_LOG_ALERT,
+	[VMM_LOGLEVEL_CRITICAL]  = VMM_LOG_CRITICAL ,
+	[VMM_LOGLEVEL_ERROR]     = VMM_LOG_ERROR,
+	[VMM_LOGLEVEL_WARNING]   = VMM_LOG_WARNING,
+	[VMM_LOGLEVEL_NOTICE]    = VMM_LOG_NOTICE,
+	[VMM_LOGLEVEL_INFO]      = VMM_LOG_INFO,
+};
+
 struct vmm_stdio_ctrl {
 	atomic_t loglevel;
         vmm_spinlock_t lock;
@@ -49,6 +103,13 @@ struct vmm_stdio_ctrl {
 static struct vmm_stdio_ctrl stdio_ctrl;
 static bool stdio_init_done = FALSE;
 
+
+static inline void _vmm_format_error(const char *fmt, char err)
+{
+	vmm_printf("\n*** vmm_stdio: invalid specifier within format "
+			"\"%%%s\": '%c' (0x%x)\n", fmt, err, err);
+}
+
 bool vmm_iscontrol(char c)
 {
 	return ((0 <= c) && (c < 32)) ? TRUE : FALSE;
@@ -240,14 +301,14 @@ static int print(char **out, u32 *out_len, struct vmm_chardev *cdev,
 			width = flags = 0;
 			if (*format == '\0')
 				break;
-			if (*format == '%')
+			else if (*format == '%')
 				goto out;
 			/* Get flags */
-			if (*format == '-') {
+			else if (*format == '-') {
 				++format;
 				flags = PAD_RIGHT;
 			}
-			if (*format == '#') {
+			else if (*format == '#') {
 				++format;
 				flags |= PAD_ALTERNATE;
 			}
@@ -265,66 +326,61 @@ static int print(char **out, u32 *out_len, struct vmm_chardev *cdev,
 				acnt += sizeof(char *);
 				pc += prints(out, out_len, cdev,
 					     s ? s : "(null)", width, flags);
-				continue;
-			}
-			if (*format == 'd') {
+			} else if ((*format == 'd') || (*format == 'i')) {
 				pc += printi(out, out_len, cdev,
 					va_arg(args, int),
 					10, 1, width, flags, '0');
 				acnt += sizeof(int);
-				continue;
-			}
-			if (*format == 'x') {
+			} else if (*format == 'x') {
 				pc += printi(out, out_len, cdev,
 					va_arg(args, unsigned int),
 					16, 0, width, flags, 'a');
 				acnt += sizeof(unsigned int);
-				continue;
-			}
-			if (*format == 'X') {
+			} else if (*format == 'X') {
 				pc += printi(out, out_len, cdev,
 					va_arg(args, unsigned int),
 					16, 0, width, flags, 'A');
 				acnt += sizeof(unsigned int);
-				continue;
-			}
-			if (*format == 'u') {
+			} else if (*format == 'u') {
 				pc += printi(out, out_len, cdev,
 					va_arg(args, unsigned int),
 					10, 0, width, flags, 'a');
 				acnt += sizeof(unsigned int);
-				continue;
-			}
-			if (*format == 'p') {
+			} else if (*format == 'p') {
 				pc += printi(out, out_len, cdev,
 					va_arg(args, unsigned long),
 					16, 0, width, flags, 'a');
 				acnt += sizeof(unsigned long);
-				continue;
-			}
-			if (*format == 'P') {
-				pc += printi(out, out_len, cdev,
-					va_arg(args, unsigned long),
-					16, 0, width, flags, 'A');
-				acnt += sizeof(unsigned long);
-				continue;
-			}
-			if (*format == 'l' && *(format + 1) == 'l') {
-				while (acnt & (sizeof(unsigned long long)-1)) {
-					va_arg(args, int);
-					acnt += sizeof(int);
-				}
-				if (sizeof(unsigned long long) ==
-						sizeof(unsigned long)) {
-					tmp = va_arg(args, unsigned long long);
-					acnt += sizeof(unsigned long long);
-				} else {
-					((unsigned long *)&tmp)[0] =
-						va_arg(args, unsigned long);
-					((unsigned long *)&tmp)[1] =
-						va_arg(args, unsigned long);
-					acnt += 2*sizeof(unsigned long);
+			} else if (*format == 'z') {
+				if (*(format + 1) == 'x') {
+					format += 1;
+					pc += printi(out, out_len, cdev,
+						va_arg(args, size_t),
+						16, 0, width, flags, 'a');
+					acnt += sizeof(size_t);
+				} else if (*(format + 1) == 'X') {
+					format += 1;
+					pc += printi(out, out_len, cdev,
+						va_arg(args, size_t),
+						16, 0, width, flags, 'A');
+					acnt += sizeof(size_t);
+				} else if (*(format + 1) == 'u') {
+					format += 1;
+					pc += printi(out, out_len, cdev,
+						va_arg(args, size_t),
+						10, 0, width, flags, 'a');
+					acnt += sizeof(size_t);
+				} else if ((*(format + 1) == 'd') || (*(format + 1) == 'i')) {
+					format += 1;
+					pc += printi(out, out_len, cdev,
+						va_arg(args, ssize_t),
+						10, 1, width, flags, '0');
+					acnt += sizeof(ssize_t);
+				} else { /* Unhandled cases */
+					_vmm_format_error("z", *(format + 1));
 				}
+			} else if (*format == 'l' && *(format + 1) == 'l') {
+				tmp = va_arg(args, unsigned long long);
 				if (*(format + 2) == 'u') {
 					format += 2;
 					pc += printi(out, out_len, cdev, tmp,
@@ -337,12 +393,13 @@ static int print(char **out, u32 *out_len, struct vmm_chardev *cdev,
 					format += 2;
 					pc += printi(out, out_len, cdev, tmp,
 						16, 0, width, flags, 'A');
-				} else {
-					format += 1;
+				} else if ((*(format + 2) == 'd') || (*(format + 2) == 'i')) {
+					format += 2;
 					pc += printi(out, out_len, cdev, tmp,
 						10, 1, width, flags, '0');
+				} else { /* Unhandled cases */
+					_vmm_format_error("ll", *(format + 2));
 				}
-				continue;
 			} else if (*format == 'l') {
 				if (*(format + 1) == 'x') {
 					format += 1;
@@ -362,20 +419,23 @@ static int print(char **out, u32 *out_len, struct vmm_chardev *cdev,
 						va_arg(args, unsigned long),
 						10, 0, width, flags, 'a');
 					acnt += sizeof(unsigned long);
-				} else {
+				} else if ((*(format + 1) == 'd') || (*(format + 1) == 'i')) {
+					format += 1;
 					pc += printi(out, out_len, cdev,
 						va_arg(args, long),
 						10, 1, width, flags, '0');
 					acnt += sizeof(long);
+				} else { /* Unhandled cases */
+					_vmm_format_error("l", *(format + 1));
 				}
-			}
-			if (*format == 'c') {
+			} else if (*format == 'c') {
 				/* char are converted to int then pushed on the stack */
 				scr[0] = va_arg(args, int);
 				scr[1] = '\0';
 				pc += prints(out, out_len, cdev, scr, width, flags);
 				acnt += sizeof(int);
-				continue;
+			} else {
+				_vmm_format_error("", *format);
 			}
 		} else {
 out:
@@ -408,12 +468,69 @@ int vmm_snprintf(char *out, u32 out_sz, const char *format, ...)
 	return retval;
 }
 
+static int vmm_cvprintf(struct vmm_chardev *cdev,
+			const char *format, va_list args)
+{
+	return print(NULL, NULL, (cdev) ? cdev : stdio_ctrl.dev, format, args);
+}
+
 int vmm_cprintf(struct vmm_chardev *cdev, const char *format, ...)
 {
 	va_list args;
 	int retval;
 	va_start(args, format);
-	retval = print(NULL, NULL, (cdev) ? cdev : stdio_ctrl.dev, format, args);
+	retval = vmm_cvprintf(cdev, format, args);
+	va_end(args);
+	return retval;
+}
+
+void vmm_chexdump(struct vmm_chardev *cdev,
+		  u64 print_base_addr, void *data, u64 len)
+{
+	u64 i;
+
+	if (!data || !len) {
+		return;
+	}
+
+	for (i = 0; i < len; i++) {
+		if ((i & 0xF) == 0) {
+			vmm_cprintf(cdev, "%016"PRIx64"  %02x",
+				    (print_base_addr + i), ((u8 *)data)[i]);
+		} else if ((i & 0xF) == 0x8) {
+			vmm_cprintf(cdev, "  %02x", ((u8 *)data)[i]);
+		} else if ((i & 0xF) == 0xF) {
+			vmm_cprintf(cdev, " %02x\n", ((u8 *)data)[i]);
+		} else {
+			vmm_cprintf(cdev, " %02x", ((u8 *)data)[i]);
+		}
+	}
+}
+
+static int vmm_lvprintf(enum vmm_print_level level,
+			const char *prefix, const char *format, va_list args)
+{
+	int retval = 0;
+	struct vmm_chardev *const cdev = stdio_ctrl.dev;
+
+	if (vmm_stdio_loglevel() >= level) {
+		retval = vmm_cprintf(cdev, "%s%s %s%s",
+				     _log_prefixes[level],
+				     VMM_LOG_COLOR_RESET,
+				     (prefix) ? prefix : "",
+				     (prefix) ? ": " : "");
+		retval += vmm_cvprintf(cdev, format, args);
+	}
+	return retval;
+}
+
+int vmm_lprintf(enum vmm_print_level level,
+		const char *prefix, const char *format, ...)
+{
+	int retval;
+	va_list args;
+	va_start(args, format);
+	retval = vmm_lvprintf(level, prefix, format, args);
 	va_end(args);
 	return retval;
 }
@@ -421,8 +538,9 @@ int vmm_cprintf(struct vmm_chardev *cdev, const char *format, ...)
 void __noreturn __vmm_panic(const char *format, ...)
 {
 	va_list args;
+
 	va_start(args, format);
-	print(NULL, NULL, stdio_ctrl.dev, format, args);
+	vmm_lvprintf(VMM_LOGLEVEL_EMERGENCY, NULL, format, args);
 	va_end(args);
 	vmm_hang();
 }
