@@ -24,7 +24,6 @@
 #include <vmm_error.h>
 #include <vmm_smp.h>
 #include <vmm_percpu.h>
-#include <vmm_spinlocks.h>
 #include <vmm_heap.h>
 #include <vmm_stdio.h>
 #include <vmm_clocksource.h>
@@ -41,6 +40,7 @@ struct vmm_timer_local_ctrl {
 	bool inprocess;
 	u64 next_event;
 	struct vmm_timer_event *curr;
+	vmm_rwlock_t event_list_lock;
 	struct dlist event_list;
 };
 
@@ -49,7 +49,7 @@ static DEFINE_PER_CPU(struct vmm_timer_local_ctrl, tlc);
 #if defined(CONFIG_PROFILE)
 u64 __notrace vmm_timer_timestamp_for_profile(void)
 {
-	return vmm_timecounter_read_for_profile(&(this_cpu(tlc).tc));
+	return vmm_timecounter_read_for_profile(&this_cpu(tlc).tc);
 }
 #endif
 
@@ -59,20 +59,20 @@ u64 vmm_timer_timestamp(void)
 	irq_flags_t flags;
 
 	arch_cpu_irq_save(flags);
-	ret = vmm_timecounter_read(&(this_cpu(tlc).tc));
+	ret = vmm_timecounter_read(&this_cpu(tlc).tc);
 	arch_cpu_irq_restore(flags);
 
 	return ret;
 }
 
-static void vmm_timer_schedule_next_event(void)
+/* Note: This function must be called with tlcp->event_list_lock held. */
+static void __timer_schedule_next_event(struct vmm_timer_local_ctrl *tlcp)
 {
 	u64 tstamp;
 	struct vmm_timer_event *e;
-	struct vmm_timer_local_ctrl *tlcp = &this_cpu(tlc);
 
 	/* If not started yet or still processing events then we give up */
-	if (!tlcp->started || tlcp->inprocess) {
+	if ((tlcp->started == FALSE) || (tlcp->inprocess == TRUE)) {
 		return;
 	}
 
@@ -84,7 +84,7 @@ static void vmm_timer_schedule_next_event(void)
 	/* Retrieve first event from list of active events */
 	e = list_entry(list_first(&tlcp->event_list), 
 		       struct vmm_timer_event,
-		       head);
+		       active_head);
 
 	/* Configure clockevent device for first event */
 	tlcp->curr = e;
@@ -99,32 +99,60 @@ static void vmm_timer_schedule_next_event(void)
 	}
 }
 
-/**
- * This is call from interrupt context. So we don't need to protect the list
- * when manipulating it.
+/* Note: This function must be called with ev->active_lock held. */
+static void __timer_event_stop(struct vmm_timer_event *ev)
+{
+	irq_flags_t flags;
+	struct vmm_timer_local_ctrl *tlcp;
+
+	if (!ev->active_state) {
+		return;
+	}
+
+	tlcp = &per_cpu(tlc, ev->active_hcpu);
+
+	vmm_write_lock_irqsave_lite(&tlcp->event_list_lock, flags);
+
+	ev->active_state = FALSE;
+	list_del(&ev->active_head);
+	ev->expiry_tstamp = 0;
+
+	vmm_write_unlock_irqrestore_lite(&tlcp->event_list_lock, flags);
+}
+
+/* This is called from interrupt context. We need to protect the 
+ * event list when manipulating it.
  */
 static void timer_clockchip_event_handler(struct vmm_clockchip *cc)
 {
+	irq_flags_t flags, flags1;
 	struct vmm_timer_event *e;
 	struct vmm_timer_local_ctrl *tlcp = &this_cpu(tlc);
 
+	vmm_read_lock_irqsave_lite(&tlcp->event_list_lock, flags);
+
 	tlcp->inprocess = TRUE;
 
-	/* process expired active events */
+	/* Process expired active events */
 	while (!list_empty(&tlcp->event_list)) {
 		e = list_entry(list_first(&tlcp->event_list),
-			       struct vmm_timer_event, head);
+			       struct vmm_timer_event, active_head);
 		/* Current timestamp */
 		if (e->expiry_tstamp <= vmm_timer_timestamp()) {
+			/* Unlock event list for processing expired event */
+			vmm_read_unlock_irqrestore_lite(&tlcp->event_list_lock, flags);
 			/* Set current CPU event to NULL */
 			tlcp->curr = NULL;
-			/* consume expired active events */
-			list_del(&e->head);
-			e->expiry_tstamp = 0;
-			e->active = FALSE;
+			/* Stop expired active event */
+			vmm_spin_lock_irqsave_lite(&e->active_lock, flags1);
+			__timer_event_stop(e);
+			vmm_spin_unlock_irqrestore_lite(&e->active_lock, flags1);
+			/* Call event handler */
 			e->handler(e);
+			/* Lock back event list */
+			vmm_read_lock_irqsave_lite(&tlcp->event_list_lock, flags);
 		} else {
-			/* no more expired events */
+			/* No more expired events */
 			break;
 		}
 	}
@@ -132,55 +160,89 @@ static void timer_clockchip_event_handler(struct vmm_clockchip *cc)
 	tlcp->inprocess = FALSE;
 
 	/* Schedule next timer event */
-	vmm_timer_schedule_next_event();
+	__timer_schedule_next_event(tlcp);
+
+	vmm_read_unlock_irqrestore_lite(&tlcp->event_list_lock, flags);
+}
+
+bool vmm_timer_event_pending(struct vmm_timer_event *ev)
+{
+	bool ret;
+	irq_flags_t flags;
+
+	if (!ev) {
+		return FALSE;
+	}
+
+	vmm_spin_lock_irqsave_lite(&ev->active_lock, flags);
+	ret = ev->active_state;
+	vmm_spin_unlock_irqrestore_lite(&ev->active_lock, flags);
+
+	return ret;
+}
+
+u64 vmm_timer_event_expiry_time(struct vmm_timer_event *ev)
+{
+	u64 exp_time;
+	irq_flags_t flags;
+
+	if (!ev) {
+		return FALSE;
+	}
+
+	vmm_spin_lock_irqsave_lite(&ev->active_lock, flags);
+	exp_time = ev->expiry_tstamp;
+	vmm_spin_unlock_irqrestore_lite(&ev->active_lock, flags);
+
+	return exp_time;
 }
 
 int vmm_timer_event_start(struct vmm_timer_event *ev, u64 duration_nsecs)
 {
-	bool added;
-	irq_flags_t flags;
-	struct dlist *l;
-	struct vmm_timer_event *e;
+	u32 hcpu;
 	u64 tstamp;
-	struct vmm_timer_local_ctrl *tlcp = &this_cpu(tlc);
+	bool found_pos = FALSE;
+	irq_flags_t flags, flags1;
+	struct vmm_timer_event *e = NULL;
+	struct vmm_timer_local_ctrl *tlcp;
 
 	if (!ev) {
 		return VMM_EFAIL;
 	}
 
+	hcpu = vmm_smp_processor_id();
+	tlcp = &per_cpu(tlc, hcpu);
 	tstamp = vmm_timer_timestamp();
 
-	arch_cpu_irq_save(flags);
+	vmm_spin_lock_irqsave_lite(&ev->active_lock, flags);
 
-	if (ev->active) {
-		/*
-		 * if the timer event is already started, we remove it from
-		 * the active list because it has changed.
-		 */
-		list_del(&ev->head);
-	}
+	__timer_event_stop(ev);
 
 	ev->expiry_tstamp = tstamp + duration_nsecs;
 	ev->duration_nsecs = duration_nsecs;
-	ev->active = TRUE;
-	added = FALSE;
-	e = NULL;
-	list_for_each(l, &tlcp->event_list) {
-		e = list_entry(l, struct vmm_timer_event, head);
+	ev->active_state = TRUE;
+	ev->active_hcpu = hcpu;
+
+	vmm_write_lock_irqsave_lite(&tlcp->event_list_lock, flags1);
+
+	list_for_each_entry(e, &tlcp->event_list, active_head) {
 		if (ev->expiry_tstamp < e->expiry_tstamp) {
-			list_add_tail(&ev->head, &e->head);
-			added = TRUE;
+			found_pos = TRUE;
 			break;
 		}
 	}
 
-	if (!added) {
-		list_add_tail(&ev->head, &tlcp->event_list);
+	if (!found_pos) {
+		list_add_tail(&ev->active_head, &tlcp->event_list);
+	} else {
+		list_add_tail(&ev->active_head, &e->active_head);
 	}
 
-	vmm_timer_schedule_next_event();
+	__timer_schedule_next_event(tlcp);
 
-	arch_cpu_irq_restore(flags);
+	vmm_write_unlock_irqrestore_lite(&tlcp->event_list_lock, flags1);
+
+	vmm_spin_unlock_irqrestore_lite(&ev->active_lock, flags);
 
 	return VMM_OK;
 }
@@ -202,21 +264,20 @@ int vmm_timer_event_stop(struct vmm_timer_event *ev)
 		return VMM_EFAIL;
 	}
 
-	arch_cpu_irq_save(flags);
+	vmm_spin_lock_irqsave_lite(&ev->active_lock, flags);
 
-	ev->expiry_tstamp = 0;
+	__timer_event_stop(ev);
 
-	if (ev->active) {
-		list_del(&ev->head);
-		ev->active = FALSE;
-		vmm_timer_schedule_next_event();
-	}
-
-	arch_cpu_irq_restore(flags);
+	vmm_spin_unlock_irqrestore_lite(&ev->active_lock, flags);
 
 	return VMM_OK;
 }
 
+bool vmm_timer_started(void)
+{
+	return this_cpu(tlc).started;
+}
+
 void vmm_timer_start(void)
 {
 	u64 tstamp;
@@ -260,21 +321,25 @@ int __cpuinit vmm_timer_init(void)
 	tlcp->curr = NULL;
 
 	/* Initialize Per CPU event list */
+	INIT_RW_LOCK(&tlcp->event_list_lock);
 	INIT_LIST_HEAD(&tlcp->event_list);
 
-	/* Find suitable clockchip */
-	if (!(tlcp->cc = vmm_clockchip_find_best(vmm_cpumask_of(cpu)))) {
-		vmm_panic("%s: No clockchip for CPU%d\n", __func__, cpu);
+	/* Bind suitable clockchip to current host CPU */
+	tlcp->cc = vmm_clockchip_bind_best(cpu);
+	if (!tlcp->cc) {
+		vmm_printf("%s: No clockchip for CPU%d\n", __func__, cpu);
+		return VMM_ENODEV;
 	}
 
 	/* Update event handler of clockchip */
 	vmm_clockchip_set_event_handler(tlcp->cc, 
 					&timer_clockchip_event_handler);
 
-	if (!cpu) {
+	if (vmm_smp_is_bootcpu()) {
 		/* Find suitable clocksource */
 		if (!(cs = vmm_clocksource_best())) {
-			vmm_panic("%s: No clocksource found\n", __func__);
+			vmm_printf("%s: No clocksource found\n", __func__);
+			return VMM_ENODEV;
 		}
 
 		/* Initialize timecounter wrapper */
