@@ -138,13 +138,13 @@ static inline int _ipu_is_sync_irq(uint32_t irq)
 #define tri_cur_buf_mask(ch)	(idma_mask(ch*2) * 3)
 #define tri_cur_buf_shift(ch)	(ffs(idma_mask(ch*2)) - 1)
 
-const char *pixel_clk_0[] = {"ipu1_pclk_0", "ipu2_pclk_0"};
-const char *pixel_clk_1[] = {"ipu1_pclk_1", "ipu2_pclk_1"};
-const char *pixel_clk_0_sel[] = {"ipu1_pclk0_sel", "ipu2_pclk0_sel"};
-const char *pixel_clk_1_sel[] = {"ipu1_pclk1_sel", "ipu2_pclk1_sel"};
-const char *pixel_clk_0_div[] = {"ipu1_pclk0_div", "ipu2_pclk0_div"};
-const char *pixel_clk_1_div[] = {"ipu1_pclk1_div", "ipu2_pclk1_div"};
-const char *ipu_pixel_clk_sel[2][3] = {{"ipu1", "ipu1_di0", "ipu1_di1"},
+static const char *const pixel_clk_0[] = {"ipu1_pclk_0", "ipu2_pclk_0"};
+static const char *const pixel_clk_1[] = {"ipu1_pclk_1", "ipu2_pclk_1"};
+static const char *const pixel_clk_0_sel[] = {"ipu1_pclk0_sel", "ipu2_pclk0_sel"};
+static const char *const pixel_clk_1_sel[] = {"ipu1_pclk1_sel", "ipu2_pclk1_sel"};
+static const char *const pixel_clk_0_div[] = {"ipu1_pclk0_div", "ipu2_pclk0_div"};
+static const char *const pixel_clk_1_div[] = {"ipu1_pclk1_div", "ipu2_pclk1_div"};
+static const char *const ipu_pixel_clk_sel[2][3] = {{"ipu1", "ipu1_di0", "ipu1_di1"},
 				       {"ipu2", "ipu2_di0", "ipu2_di1"}};
 
 static int ipu_clk_setup_enable(struct ipu_soc *ipu,
@@ -159,7 +159,7 @@ static int ipu_clk_setup_enable(struct ipu_soc *ipu,
 			(const char **)ipu_pixel_clk_sel[pdata->id],
 			ARRAY_SIZE(ipu_pixel_clk_sel[pdata->id]),
 			0, pdata->id, 0, 0);
-	if (VMM_IS_ERR(clk)) {
+	if (VMM_IS_ERR_OR_NULL(clk)) {
 		dev_err(ipu->dev, "clk_register mux di0 failed");
 		return VMM_PTR_ERR(clk);
 	}
@@ -168,7 +168,7 @@ static int ipu_clk_setup_enable(struct ipu_soc *ipu,
 			(const char **)ipu_pixel_clk_sel[pdata->id],
 			ARRAY_SIZE(ipu_pixel_clk_sel[pdata->id]),
 			0, pdata->id, 1, 0);
-	if (VMM_IS_ERR(clk)) {
+	if (VMM_IS_ERR_OR_NULL(clk)) {
 		dev_err(ipu->dev, "clk_register mux di1 failed");
 		return VMM_PTR_ERR(clk);
 	}
@@ -177,14 +177,14 @@ static int ipu_clk_setup_enable(struct ipu_soc *ipu,
 	clk = clk_register_div_pix_clk(ipu->dev, pixel_clk_0_div[pdata->id],
 				       pixel_clk_0_sel[pdata->id], 0,
 				       pdata->id, 0, 0);
-	if (VMM_IS_ERR(clk)) {
+	if (VMM_IS_ERR_OR_NULL(clk)) {
 		dev_err(ipu->dev, "clk register di0 div failed");
 		return VMM_PTR_ERR(clk);
 	}
 	clk = clk_register_div_pix_clk(ipu->dev, pixel_clk_1_div[pdata->id],
 				       pixel_clk_1_sel[pdata->id],
 				       CLK_SET_RATE_PARENT, pdata->id, 1, 0);
-	if (VMM_IS_ERR(clk)) {
+	if (VMM_IS_ERR_OR_NULL(clk)) {
 		dev_err(ipu->dev, "clk register di1 div failed");
 		return VMM_PTR_ERR(clk);
 	}
@@ -194,7 +194,7 @@ static int ipu_clk_setup_enable(struct ipu_soc *ipu,
 						      pixel_clk_0_div[pdata->id],
 						      CLK_SET_RATE_PARENT,
 						      pdata->id, 0, 0);
-	if (VMM_IS_ERR(ipu->pixel_clk[0])) {
+	if (VMM_IS_ERR_OR_NULL(ipu->pixel_clk[0])) {
 		dev_err(ipu->dev, "clk register di0 gate failed");
 		return VMM_PTR_ERR(ipu->pixel_clk[0]);
 	}
@@ -204,7 +204,7 @@ static int ipu_clk_setup_enable(struct ipu_soc *ipu,
 					  pixel_clk_1_div[pdata->id],
 					  CLK_SET_RATE_PARENT,
 					  pdata->id, 1, 0);
-	if (VMM_IS_ERR(ipu->pixel_clk[1])) {
+	if (VMM_IS_ERR_OR_NULL(ipu->pixel_clk[1])) {
 		dev_err(ipu->dev, "clk register di1 gate failed");
 		return VMM_PTR_ERR(ipu->pixel_clk[1]);
 	}
@@ -222,23 +222,23 @@ static int ipu_clk_setup_enable(struct ipu_soc *ipu,
 	}
 
 	ipu->di_clk[0] = devm_clk_get(ipu->dev, "di0");
-	if (VMM_IS_ERR(ipu->di_clk[0])) {
+	if (VMM_IS_ERR_OR_NULL(ipu->di_clk[0])) {
 		dev_err(ipu->dev, "clk_get di0 failed");
 		return VMM_PTR_ERR(ipu->di_clk[0]);
 	}
 	ipu->di_clk[1] = devm_clk_get(ipu->dev, "di1");
-	if (VMM_IS_ERR(ipu->di_clk[1])) {
+	if (VMM_IS_ERR_OR_NULL(ipu->di_clk[1])) {
 		dev_err(ipu->dev, "clk_get di1 failed");
 		return VMM_PTR_ERR(ipu->di_clk[1]);
 	}
 
 	ipu->di_clk_sel[0] = devm_clk_get(ipu->dev, "di0_sel");
-	if (VMM_IS_ERR(ipu->di_clk_sel[0])) {
+	if (VMM_IS_ERR_OR_NULL(ipu->di_clk_sel[0])) {
 		dev_err(ipu->dev, "clk_get di0_sel failed");
 		return VMM_PTR_ERR(ipu->di_clk_sel[0]);
 	}
 	ipu->di_clk_sel[1] = devm_clk_get(ipu->dev, "di1_sel");
-	if (VMM_IS_ERR(ipu->di_clk_sel[1])) {
+	if (VMM_IS_ERR_OR_NULL(ipu->di_clk_sel[1])) {
 		dev_err(ipu->dev, "clk_get di1_sel failed");
 		return VMM_PTR_ERR(ipu->di_clk_sel[1]);
 	}
@@ -334,7 +334,7 @@ static int ipu_probe(struct vmm_device *dev,
 	if (!pltfm_data)
 		return VMM_ENOMEM;
 
-	ret = vmm_devtree_read_u32(dev->node, "bypass_reset", &bypass_reset);
+	ret = vmm_devtree_read_u32(dev->of_node, "bypass_reset", &bypass_reset);
 	if (ret < 0) {
 		dev_dbg(dev, "can not get bypass_reset\n");
 		return ret;
@@ -342,6 +342,10 @@ static int ipu_probe(struct vmm_device *dev,
 	pltfm_data->bypass_reset = (bool)bypass_reset;
 
 	/* Aliases are not yet implemented, use a dirty static int */
+	if (id >= MXC_IPU_MAX_NUM) {
+		dev_err(dev, "id overflow (%"PRIu32")\n", id);
+		return VMM_EOVERFLOW;
+	}
 	pltfm_data->id = id;
 	++id;
 
@@ -358,19 +362,19 @@ static int ipu_probe(struct vmm_device *dev,
 	spin_lock_init(&ipu->rdy_reg_spin_lock);
 	mutex_init(&ipu->mutex_lock);
 
-	ret = vmm_devtree_irq_get(dev->node, &ipu->irq_sync, 0);
-	if (ret) {
+	ipu->irq_sync = vmm_devtree_irq_parse_map(dev->of_node, 0);
+	if (!ipu->irq_sync) {
 		dev_err(ipu->dev, "request SYNC interrupt failed\n");
-		return ret;
+		return VMM_ENODEV;
 	}
 
-	ret = vmm_devtree_irq_get(dev->node, &ipu->irq_err, 1);
-	if (ret) {
+	ipu->irq_err = vmm_devtree_irq_parse_map(dev->of_node, 1);
+	if (!ipu->irq_err) {
 		dev_err(ipu->dev, "request ERR interrupt failed\n");
-		return ret;
+		return VMM_ENODEV;
 	}
 
-	ret = vmm_devtree_regaddr(dev->node, &ipu_base, 0);
+	ret = vmm_devtree_regaddr(dev->of_node, &ipu_base, 0);
 	if (ret) {
 		dev_err(dev, "can't get device resources\n");
 		return ret;
@@ -443,9 +447,9 @@ static int ipu_probe(struct vmm_device *dev,
 	dev_dbg(ipu->dev, "IPU VDI Regs = %p\n", ipu->vdi_reg);
 
 	ipu->ipu_clk = devm_clk_get(ipu->dev, "bus");
-	if (VMM_IS_ERR(ipu->ipu_clk)) {
+	if (VMM_IS_ERR_OR_NULL(ipu->ipu_clk)) {
 		dev_err(ipu->dev, "clk_get ipu failed");
-		return PTR_ERR(ipu->ipu_clk);
+		return VMM_PTR_ERR(ipu->ipu_clk);
 	}
 
 	/* ipu_clk is always prepared */
@@ -2518,7 +2522,7 @@ VMM_EXPORT_SYMBOL(ipu_disable_channel);
 int32_t ipu_channel_disable(struct ipu_chan *ipu_chan, bool wait_for_stop)
 {
 	if (ipu_chan)
-		if (!VMM_IS_ERR(ipu_chan))
+		if (!VMM_IS_ERR_OR_NULL(ipu_chan))
 			return ipu_disable_channel(ipu_chan->ipu, ipu_chan->channel, wait_for_stop);
 	return 0;
 }
