@@ -11,6 +11,12 @@
 #ifndef __LINUX_CLK_H
 #define __LINUX_CLK_H
 
-/* Note: This file is just a place holder */
+#include <drv/clk.h>
+
+#include <linux/err.h>
+#include <linux/kernel.h>
+#include <linux/notifier.h>
+
+/* For now this is just place holder header. */
 
 #endif
