@@ -178,11 +178,11 @@ static int sdhci_transfer_dma(struct sdhci_host *host,
 			      struct mmc_data *data)
 {
 	int rc = VMM_OK;
-	u64 timeout = 100000000;
+	u64 timeout = 100000000LL;
 
 	rc = vmm_completion_wait_timeout(&host->wait_dma, &timeout);
 	if (VMM_ETIMEDOUT == rc) {
-		vmm_printf("%s: Transfer data timeout (%d)\n", __func__,
+		vmm_printf("%s: Transfer data timeout (%"PRId64")\n", __func__,
 			   timeout);
 		return rc;
 	}
@@ -193,7 +193,7 @@ static int sdhci_transfer_dma(struct sdhci_host *host,
 static void sdhci_transfer_pio(struct sdhci_host *host, struct mmc_data *data)
 {
 	int i;
-	char *offs;
+	u8 *offs;
 	for (i = 0; i < data->blocksize; i += 4) {
 		offs = data->dest + i;
 		if (data->flags == MMC_DATA_READ) {
@@ -354,9 +354,10 @@ int sdhci_send_command(struct mmc_host *mmc,
 			sdhci_writel(host, (u32)dma_addr, SDHCI_DMA_ADDRESS);
 			mode |= SDHCI_TRNS_DMA;
 
-			vmm_flush_cache_range((u32)host->aligned_buffer,
-					      (u32)host->aligned_buffer +
-					      trans_bytes);
+			vmm_flush_cache_range(
+					(virtual_addr_t)host->aligned_buffer,
+					(virtual_addr_t)host->aligned_buffer +
+					trans_bytes);
 		}
 
 		sdhci_writew(host, SDHCI_MAKE_BLKSZ(SDHCI_DEFAULT_BOUNDARY_ARG,
@@ -421,9 +422,9 @@ int sdhci_send_command(struct mmc_host *mmc,
 		if (host->sdhci_caps & SDHCI_CAN_DO_SDMA) {
 			ret = sdhci_transfer_dma(host, data);
 		} else {
-			u32 start_addr = (u32)data->dest;
+			u32 start_addr = (virtual_addr_t)data->dest;
 			if (data->flags != MMC_DATA_READ) {
-				start_addr = (u32)data->src;
+				start_addr = (virtual_addr_t)data->src;
 			}
 			ret = sdhci_transfer_data(host, data, start_addr);
 		}
@@ -469,7 +470,7 @@ static int sdhci_set_clock(struct mmc_host *mmc, u32 clock)
 
 	sdhci_writew(host, 0, SDHCI_CLOCK_CONTROL);
 
-	if ((host->sdhci_version & SDHCI_SPEC_VER_MASK) >= SDHCI_SPEC_300) {
+	if (SDHCI_GET_VERSION(host) >= SDHCI_SPEC_300) {
 		/* Version 3.00 divisors must be a multiple of 2. */
 		if (mmc->f_max <= clock)
 			div = 1;
@@ -565,13 +566,11 @@ static void sdhci_set_ios(struct mmc_host *mmc, struct mmc_ios *ios)
 	ctrl = sdhci_readb(host, SDHCI_HOST_CONTROL);
 	if (ios->bus_width == 8) {
 		ctrl &= ~SDHCI_CTRL_4BITBUS;
-		if ((host->sdhci_version & SDHCI_SPEC_VER_MASK) >=
-							SDHCI_SPEC_300) {
+		if (SDHCI_GET_VERSION(host) >= SDHCI_SPEC_300) {
 			ctrl |= SDHCI_CTRL_8BITBUS;
 		}
 	} else {
-		if ((host->sdhci_version & SDHCI_SPEC_VER_MASK) >=
-							SDHCI_SPEC_300) {
+		if (SDHCI_GET_VERSION(host) >= SDHCI_SPEC_300) {
 			ctrl &= ~SDHCI_CTRL_8BITBUS;
 		}
 		if (ios->bus_width == 4) {
@@ -605,7 +604,7 @@ static int sdhci_get_cd(struct mmc_host *mmc)
 		return VMM_ENOTSUPP;
 
 	/* Try slot gpio detect */
-	if (!gpio_cd >= 0)
+	if (!(gpio_cd >= 0))
 		return !!gpio_cd;
 
 	/* Host native card detect */
@@ -802,7 +801,7 @@ int sdhci_add_host(struct sdhci_host *host)
 	if (host->max_clk) {
 		mmc->f_max = host->max_clk;
 	} else {
-		if ((host->sdhci_version & SDHCI_SPEC_VER_MASK) >= SDHCI_SPEC_300) {
+		if (SDHCI_GET_VERSION(host) >= SDHCI_SPEC_300) {
 			mmc->f_max = (host->sdhci_caps & SDHCI_CLOCK_V3_BASE_MASK)
 					>> SDHCI_CLOCK_BASE_SHIFT;
 		} else {
@@ -819,7 +818,7 @@ int sdhci_add_host(struct sdhci_host *host)
 	if (host->min_clk) {
 		mmc->f_min = host->min_clk;
 	} else {
-		if ((host->sdhci_version & SDHCI_SPEC_VER_MASK) >= SDHCI_SPEC_300) {
+		if (SDHCI_GET_VERSION(host) >= SDHCI_SPEC_300) {
 			mmc->f_min = mmc->f_max / SDHCI_MAX_DIV_SPEC_300;
 		} else {
 			mmc->f_min = mmc->f_max / SDHCI_MAX_DIV_SPEC_200;
@@ -844,10 +843,15 @@ int sdhci_add_host(struct sdhci_host *host)
 	mmc->caps = MMC_CAP_MODE_HS |
 		    MMC_CAP_MODE_HS_52MHz |
 		    MMC_CAP_MODE_4BIT;
-	if (host->sdhci_caps & SDHCI_CAN_DO_8BIT) {
-		mmc->caps |= MMC_CAP_MODE_8BIT;
+	if (SDHCI_GET_VERSION(host) >= SDHCI_SPEC_300) {
+		if (host->sdhci_caps & SDHCI_CAN_DO_8BIT) {
+			mmc->caps |= MMC_CAP_MODE_8BIT;
+		}
 	}
 
+	if (host->quirks & SDHCI_QUIRK_NO_HISPD_BIT)
+		mmc->caps &= ~(MMC_CAP_MODE_HS | MMC_CAP_MODE_HS_52MHz);
+
 	if (host->quirks & SDHCI_QUIRK_BROKEN_CARD_DETECTION) {
 		mmc->caps |= MMC_CAP_NEEDS_POLL;
 	}
@@ -876,7 +880,7 @@ int sdhci_add_host(struct sdhci_host *host)
 			goto free_nothing;
 		}
 		if ((host->quirks & SDHCI_QUIRK_32BIT_DMA_ADDR) &&
-		    (((u32)host->aligned_buffer) & 0x7)) {
+		    (((virtual_addr_t)host->aligned_buffer) & 0x7)) {
 			vmm_printf("%s: host buffer not aligned to "
 				   "8-byte boundary!!!\n", __func__);
 			rc = VMM_EFAIL;
@@ -900,7 +904,7 @@ int sdhci_add_host(struct sdhci_host *host)
 		goto free_host_irq;
 	}
 
-	switch (host->sdhci_version & SDHCI_SPEC_VER_MASK) {
+	switch (SDHCI_GET_VERSION(host)) {
 	case SDHCI_SPEC_100:
 		ver = "v1";
 		break;
