@@ -22,10 +22,12 @@
  * @brief implementation of string library
  */
 
+#include <vmm_error.h>
 #include <vmm_types.h>
 #include <vmm_host_io.h>
 #include <vmm_limits.h>
 #include <vmm_compiler.h>
+#include <vmm_stdio.h>
 #include <vmm_modules.h>
 #include <libs/ctype.h>
 #include <libs/stringlib.h>
@@ -151,6 +153,19 @@ int strcasecmp(const char *s1, const char *s2)
 	return c1 - c2;
 }
 
+int strncasecmp(const char *a, const char *b, size_t n)
+{
+	if (n > 0) {
+		/* search first diff or end of string */
+		for (n--;
+		     n != 0 && tolower(*a) == tolower(*b) && *a != '\0';
+		     a++, b++, n--);
+		return tolower(*a) - tolower(*b);
+	}
+
+	return 0;
+}
+
 char *strchr(const char *s, int c)
 {
 	/* search for the c char starting left */
@@ -403,31 +418,6 @@ char *strsep(char **s, const char *ct)
 	return sbegin;
 }
 
-#if !defined(ARCH_HAS_MEMCPY)
-void *memcpy(void *dest, const void *src, size_t count)
-{
-	u8 *dst8 = (u8 *) dest;
-	u8 *src8 = (u8 *) src;
-
-	if (count & 1) {
-		dst8[0] = src8[0];
-		dst8 += 1;
-		src8 += 1;
-	}
-
-	count /= 2;
-	while (count--) {
-		dst8[0] = src8[0];
-		dst8[1] = src8[1];
-
-		dst8 += 2;
-		src8 += 2;
-	}
-
-	return dest;
-}
-#endif
-
 void *memcpy_toio(void *dest, const void *src, size_t count)
 {
 	u8 *dst8 = (u8 *) dest;
@@ -660,6 +650,62 @@ char* strtok_r(char *str, const char *delim, char **context)
 	return ret;
 }
 
+int u64_to_size_str(u64 val, char *out, size_t out_len)
+{
+	const char *suffix = "";
+
+	if (!out || (out_len < 16))
+		return VMM_EINVALID;
+
+	if (val < SZ_1K) {
+		suffix = "B";
+		goto found;
+	}
+	val = val >> 10;
+
+	if (val < SZ_1K) {
+		suffix = "KB";
+		goto found;
+	}
+	val = val >> 10;
+
+	if (val < SZ_1K) {
+		suffix = "MB";
+		goto found;
+	}
+	val = val >> 10;
+
+	if (val < SZ_1K) {
+		suffix = "GB";
+		goto found;
+	}
+	val = val >> 10;
+
+	if (val < SZ_1K) {
+		suffix = "TB";
+		goto found;
+	}
+	val = val >> 10;
+
+	if (val < SZ_1K) {
+		suffix = "PB";
+		goto found;
+	}
+	val = val >> 10;
+
+	if (val < SZ_1K) {
+		suffix = "EB";
+		goto found;
+	}
+
+	suffix = "ZB";
+
+found:
+	vmm_snprintf(out, out_len, "%"PRIu64" %s", val, suffix);
+
+	return VMM_OK;
+}
+
 /**
  * vsscanf - Unformat a buffer into a list of arguments
  * @buf:	input buffer
