@@ -30,9 +30,11 @@ void arm_board_init(void);
 char *arm_board_name(void);
 u32 arm_board_ram_start(void);
 u32 arm_board_ram_size(void);
-u32 arm_board_flash_addr(void);
+u32 arm_board_autoexec_addr(void);
+u32 arm_board_boot_delay(void);
 u32 arm_board_linux_machine_type(void);
 void arm_board_linux_default_cmdline(char *cmdline, u32 cmdline_sz);
+void arm_board_fdt_fixup(void *fdt_addr);
 
 u32 arm_board_iosection_count(void);
 physical_addr_t arm_board_iosection_addr(int num);
@@ -45,8 +47,17 @@ int arm_board_pic_eoi_irq(u32 irq);
 int arm_board_pic_mask(u32 irq);
 int arm_board_pic_unmask(u32 irq);
 
+void arm_board_timer_enable(void);
+void arm_board_timer_disable(void);
+u64 arm_board_timer_irqcount(void);
+u64 arm_board_timer_irqdelay(void);
+u64 arm_board_timer_timestamp(void);
+void arm_board_timer_change_period(u32 usecs);
+int arm_board_timer_init(u32 usecs);
+
 int arm_board_serial_init(void);
 void arm_board_serial_putc(char ch);
+bool arm_board_serial_can_getc(void);
 char arm_board_serial_getc(void);
 
 #endif
