@@ -50,6 +50,8 @@ void arm_init(void)
 
 	arm_timer_init(10000);
 
+	arm_board_init();
+
 	memory_size = arm_board_ram_size();
 
 	arm_timer_enable();
@@ -107,6 +109,9 @@ void arm_cmd_help(int argc, char **argv)
 	arm_puts("            <initrd_addr>  = initrd load address (optional)\n");
 	arm_puts("            <initrd_size>  = initrd size (optional)\n");
 	arm_puts("\n");
+	arm_puts("fdt_override_u32 - Overrides an integer property in the device tree\n");
+	arm_puts("            Usage: fdt_override_u32 <fdt_addr> </path/to/property> <value>\n");
+	arm_puts("\n");
 #endif
 	arm_puts("linux_cmdline - Show/Update linux command line\n");
 	arm_puts("            Usage: linux_cmdline [<new_linux_cmdline>]\n");
@@ -348,7 +353,7 @@ void arm_cmd_copy(int argc, char **argv)
 {
 	u64 tstamp;
 	char time[256];
-	u32 *dest, *src;
+	void *dst, *src;
 	virtual_addr_t dest_va;
 	u32 i, count;
 
@@ -357,17 +362,9 @@ void arm_cmd_copy(int argc, char **argv)
 		arm_puts ("copy: must provide <dest>, <src>, and <count>\n");
 		return;
 	}
-	dest = (u32 *)arm_hexstr2uint(argv[1]);
-	if (((virtual_addr_t)dest) & 0x3) {
-		arm_puts ("copy: <dest> should be 4-byte aligned\n");
-		return;
-	}
-	dest_va = (virtual_addr_t)dest;
-	src = (u32 *)arm_hexstr2uint(argv[2]);
-	if (((virtual_addr_t)src) & 0x3) {
-		arm_puts ("copy: <src> should be 4-byte aligned\n");
-		return;
-	}
+	dst = (void *)arm_hexstr2uint(argv[1]);
+	dest_va = (virtual_addr_t)dst;
+	src = (void *)arm_hexstr2uint(argv[2]);
 	count = arm_hexstr2uint(argv[3]);
 
 	/* Disable timer and get start timestamp */
@@ -384,8 +381,28 @@ void arm_cmd_copy(int argc, char **argv)
 	arm_clean_invalidate_dcache_mva_range(dest_va, dest_va + count);
 
 	/* Copy contents */
-	for (i = 0; i < (count/sizeof(*dest)); i++) {
-		dest[i] = src[i];
+	if (!((virtual_addr_t)dst & 0x7) &&
+	    !((virtual_addr_t)src & 0x7) &&
+	    !(count & 0x7)) {
+		for (i = 0; i < (count/sizeof(u64)); i++) {
+			((u64 *)dst)[i] = ((u64 *)src)[i];
+		}
+	} else if (!((virtual_addr_t)dst & 0x3) &&
+		   !((virtual_addr_t)src & 0x3) &&
+		   !(count & 0x3)) {
+		for (i = 0; i < (count/sizeof(u32)); i++) {
+			((u32 *)dst)[i] = ((u32 *)src)[i];
+		}
+	} else if (!((virtual_addr_t)dst & 0x1) &&
+		   !((virtual_addr_t)src & 0x1) &&
+		   !(count & 0x1)) {
+		for (i = 0; i < (count/sizeof(u16)); i++) {
+			((u16 *)dst)[i] = ((u16 *)src)[i];
+		}
+	} else {
+		for (i = 0; i < (count/sizeof(u8)); i++) {
+			((u8 *)dst)[i] = ((u8 *)src)[i];
+		}
 	}
 
 	/* Enable timer and get end timestamp */
@@ -451,6 +468,11 @@ void arm_cmd_start_linux(int argc, char **argv)
 	nuke_va = kernel_addr & ~(0x100000 - 1);
 	arm_clean_invalidate_dcache_mva_range(nuke_va, nuke_va + 0x100000);
 
+	/* Disable interrupts, disable timer, and cleanup MMU */
+	arm_timer_disable();
+	arm_irq_disable();
+	arm_mmu_cleanup();
+
 	/* Setup kernel args */
 	for (p = 0; p < 128; p++) {
 		kernel_args[p] = 0x0;
@@ -493,10 +515,6 @@ void arm_cmd_start_linux(int argc, char **argv)
 	kernel_args[p++] = 0;
 	kernel_args[p++] = 0;
 
-	/* Disable interrupts and timer */
-	arm_timer_disable();
-	arm_irq_disable();
-
 	/* Jump to Linux Kernel
 	 * r0 -> zero
 	 * r1 -> board machine type
@@ -556,10 +574,18 @@ void arm_cmd_start_linux_fdt(int argc, char **argv)
 	nuke_va = kernel_addr & ~(0x100000 - 1);
 	arm_clean_invalidate_dcache_mva_range(nuke_va, nuke_va + 0x100000);
 
+	/* Disable interrupts, disable timer, and cleanup MMU */
+	arm_timer_disable();
+	arm_irq_disable();
+	arm_mmu_cleanup();
+
 	/* Pass memory size via kernel command line */
 	arm_sprintf(cfg_str, " mem=%dM", (memory_size >> 20));
 	arm_strcat(cmdline, cfg_str);
 
+	/* Increase fdt blob size by 8KB */
+	fdt_increase_size((void *)fdt_addr, 0x2000);
+
 	/* Fillup/fixup the fdt blob with following:
 	 * 		- initrd start, end
 	 * 		- kernel cmd line
@@ -570,10 +596,8 @@ void arm_cmd_start_linux_fdt(int argc, char **argv)
 			   initrd_addr, initrd_addr + initrd_size, 1);
 	}
 
-	/* Disable interrupts and timer */
-	arm_timer_disable();
-	arm_irq_disable();
-	arm_mmu_cleanup();
+	/* Do board specific fdt fixup */
+	arm_board_fdt_fixup((void *)fdt_addr);
 
 	/* Jump to Linux Kernel
 	 * r0 -> dtb address
@@ -586,6 +610,52 @@ void arm_cmd_start_linux_fdt(int argc, char **argv)
 
 	return;
 }
+
+void arm_cmd_fdt_override_u32(int argc, char **argv)
+{
+	const char *path;
+	u32 val;
+	int err;
+	int nodeoffset;
+	char *prop;
+	void *fdt;
+	const struct fdt_property *p;
+
+	if (argc < 4) {
+		arm_puts("fdt_override_u32: must provide <fdt_addr> </path/to/property> and <value>\n");
+		return;
+	}
+
+	fdt = (void *)arm_hexstr2uint(argv[1]);
+	path = argv[2];
+	val = cpu_to_be32(arm_str2int(argv[3]));
+	prop = arm_strrchr(path, '/');
+	if (!prop) {
+		arm_puts("*** Failed to parse node\n");
+		return;
+	}
+	*(prop++) = '\0';
+
+	nodeoffset = fdt_path_offset(fdt, path);
+	if (nodeoffset < 0) {
+		printf("*** Path \"%s\" does not exist\n", path);
+		return;
+	}
+
+	p = fdt_get_property(fdt, nodeoffset, prop, NULL);
+	if (!p) {
+		printf("*** Failed to find property \"%s\" of node \"%s\"\n",
+		       prop, path);
+		return;
+	}
+
+	err = fdt_setprop(fdt, nodeoffset, prop, &val, sizeof(u32));
+	if (err) {
+		printf("*** Failed to set property \"%s\" of node \"%s\". "
+		       "Error: %i\n", prop, path, err);
+		return;
+	}
+}
 #endif
 
 void arm_cmd_linux_cmdline(int argc, char **argv)
@@ -631,8 +701,7 @@ void arm_cmd_autoexec(int argc, char **argv)
 #define ARM_CMD_AUTOEXEC_BUF_SIZE	4096
 	static int lock = 0;
 	int len, pos = 0;
-	/* commands to execute are stored in NOR flash */
-	char *ptr = (char *)(arm_board_flash_addr() + 0xFF000);
+	char *ptr = (char *)(arm_board_autoexec_addr());
 	char buffer[ARM_CMD_AUTOEXEC_BUF_SIZE];
 
 	if (argc != 1) {
@@ -658,11 +727,11 @@ void arm_cmd_autoexec(int argc, char **argv)
 	/* sanity check on command list */
 	if (!len) {
 		arm_puts("command list not found !!!\n");
-		return;
+		goto unlock;
 	}
 	if (len >= ARM_CMD_AUTOEXEC_BUF_SIZE) {
 		arm_printf("command list len=%d too big !!!\n", len);
-		return;
+		goto unlock;
 	}
 
 	/* copy commands from NOR flash */
@@ -690,6 +759,7 @@ void arm_cmd_autoexec(int argc, char **argv)
 		arm_exec(ptr);
 	}
 
+unlock:
 	lock = 0;
 
 	return;
@@ -789,6 +859,8 @@ void arm_exec(char *line)
 #ifdef BOARD_FDT_SUPPORT
 		} else if (arm_strcmp(argv[0], "start_linux_fdt") == 0) {
 			arm_cmd_start_linux_fdt(argc, argv);
+                } else if (arm_strcmp(argv[0], "fdt_override_u32") == 0) {
+			arm_cmd_fdt_override_u32(argc, argv);
 #endif
 		} else if (arm_strcmp(argv[0], "linux_cmdline") == 0) {
 			arm_cmd_linux_cmdline(argc, argv);
@@ -811,6 +883,8 @@ void arm_exec(char *line)
 /* Works in user mode */
 void arm_main(void)
 {
+	u64 tstamp;
+	u32 i, key_pressed, boot_delay;
 	char line[ARM_MAX_CMD_STR_SIZE];
 
 	/* Setup board specific linux default cmdline */
@@ -819,7 +893,37 @@ void arm_main(void)
 	arm_puts(arm_board_name());
 	arm_puts(" Basic Firmware\n\n");
 
-	arm_board_init();
+	boot_delay = arm_board_boot_delay();
+	if (boot_delay == 0xffffffff) {
+		arm_puts("autoboot: disabled\n\n");
+	} else {
+		arm_puts("autoboot: enabled\n");
+		while (boot_delay) {
+			arm_int2str(line, (int)boot_delay);
+			arm_puts("autoboot: waiting for ");
+			arm_puts(line);
+			arm_puts(" secs (press any key)\n");
+			key_pressed = 0;
+			tstamp = arm_timer_timestamp();
+			while ((arm_timer_timestamp() - tstamp)
+						< 1000000000) {
+				for (i = 0; i < 10000; i++) ;
+				if (arm_can_getc()) {
+					arm_getc();
+					key_pressed = 1;
+					break;
+				}
+			}
+			if (key_pressed)
+				break;
+			boot_delay--;
+		}
+		arm_puts("\n");
+		if (!boot_delay) {
+			arm_strcpy(line, "autoexec\n");
+			arm_exec(line);
+		}
+	}
 
 	while(1) {
 		arm_puts("basic# ");
