@@ -47,6 +47,21 @@ bool arm_isprintable(char c)
 	return FALSE;
 }
 
+void arm_putc(char ch)
+{
+	arm_board_serial_putc(ch);
+}
+
+bool arm_can_getc(void)
+{
+	return arm_board_serial_can_getc();
+}
+
+char arm_getc(void)
+{
+	return arm_board_serial_getc();
+}
+
 void arm_puts(const char * str)
 {
 	while (*str) {
@@ -226,7 +241,7 @@ static int print(char **out, u32 *out_len, const char *format, va_list args)
 					     s ? s : "(null)", width, flags);
 				continue;
 			}
-			if (*format == 'd') {
+			if ((*format == 'd') || (*format == 'i')) {
 				pc += printi(out, out_len, 
 					va_arg(args, int), 
 					10, 1, width, flags, '0');
