@@ -28,6 +28,8 @@
 #define VIRT_V8_GIC			(0x08000000)
 #define VIRT_V8_GIC_SIZE		(0x00020000)
 #define VIRT_V8_UART0			(0x09000000)
+#define VIRT_V8_VMINFO			(0x09001000)
+#define VIRT_V8_SIMPLEFB		(0x09002000)
 #define VIRT_V8_VIRTIO_NET		(0x0A000000)
 #define VIRT_V8_VIRTIO_NET_SIZE		(0x00001000)
 #define VIRT_V8_VIRTIO_BLK		(0x0A001000)
